from typing import List

from qtpy import PYQT5, PYSIDE2

if PYQT5:
    from PyQt5 import QtWebEngineWidgets  # type: ignore
elif PYSIDE2:
    from PySide2 import QtWebEngineWidgets

from prettyqt import core
from prettyqt.utils import bidict

mod = QtWebEngineWidgets.QWebEngineContextMenuData

EDIT_FLAGS = bidict(
    undo=mod.CanUndo,
    redo=mod.CanRedo,
    cut=mod.CanCut,
    copy=mod.CanCopy,
    paste=mod.CanPaste,
    delete=mod.CanDelete,
    select_all=mod.CanSelectAll,
    translate=mod.CanTranslate,
    edit_richly=mod.CanEditRichly,
)


MEDIA_FLAGS = bidict(
    in_error=mod.MediaInError,
    paused=mod.MediaPaused,
    muted=mod.MediaMuted,
    loop=mod.MediaLoop,
    can_save=mod.MediaCanSave,
    has_audio=mod.MediaHasAudio,
    can_toggle_controls=mod.MediaCanToggleControls,
    controls=mod.MediaControls,
    can_print=mod.MediaCanPrint,
    can_rotate=mod.MediaCanRotate,
)

MEDIA_TYPES = bidict(
    none=mod.MediaTypeNone,
    image=mod.MediaTypeImage,
    video=mod.MediaTypeVideo,
    audio=mod.MediaTypeAudio,
    canvas=mod.MediaTypeCanvas,
    file=mod.MediaTypeFile,
    plugin=mod.MediaTypePlugin,
)


class WebEngineContextMenuData(QtWebEngineWidgets.QWebEngineContextMenuData):
    def get_media_url(self) -> core.Url:
        return core.Url(self.mediaUrl())

    def get_link_url(self) -> core.Url:
        return core.Url(self.linkUrl())

    def get_media_type(self) -> str:
        return MEDIA_TYPES.inverse[self.mediaType()]

    def get_media_flags(self) -> List[str]:
        return [k for k, v in MEDIA_FLAGS.items() if v & self.mediaFlags()]

    def get_edit_flags(self) -> List[str]:
        return [k for k, v in EDIT_FLAGS.items() if v & self.editFlags()]

    def can_undo(self) -> bool:
        return mod.CanUndo & self.editFlags()

    def can_redo(self) -> bool:
        return mod.CanRedo & self.editFlags()

    def can_cut(self) -> bool:
        return mod.CanCut & self.editFlags()

    def can_copy(self) -> bool:
        return mod.CanCopy & self.editFlags()

    def can_paste(self) -> bool:
        return mod.CanPaste & self.editFlags()

    def can_delete(self) -> bool:
        return mod.CanDelete & self.editFlags()

    def can_select_all(self) -> bool:
        return mod.CanSelectAll & self.editFlags()

    def can_translate(self) -> bool:
        return mod.CanTranslate & self.editFlags()

    def can_edit_richly(self) -> bool:
        return mod.CanEditRichly & self.editFlags()


if __name__ == "__main__":
    from prettyqt import widgets, webenginewidgets

    app = widgets.app()
    page = webenginewidgets.WebEnginePage()
    context_menu_data = page.get_context_menu_data()
    app.main_loop()



from qtpy import QtGui

from prettyqt import gui


QtGui.QBitmap.__bases__ = (gui.Pixmap,)


class Bitmap(QtGui.QBitmap):
    pass


if __name__ == "__main__":
    app = gui.app()
    image = Bitmap()
    bytes(image)



from __future__ import annotations

from typing import Optional, Dict, Union, Tuple, Literal

from qtpy import QtWidgets

from prettyqt import widgets
from prettyqt.utils import bidict, InvalidParamError


ROLE = bidict(
    left=QtWidgets.QFormLayout.LabelRole,
    right=QtWidgets.QFormLayout.FieldRole,
    both=QtWidgets.QFormLayout.SpanningRole,
)

RoleStr = Literal["left", "right", "both"]

ROW_WRAP_POLICY = bidict(
    dont_wrap=QtWidgets.QFormLayout.DontWrapRows,
    wrap_long=QtWidgets.QFormLayout.WrapLongRows,
    wrap_all=QtWidgets.QFormLayout.WrapAllRows,
)

RowWrapPolicyStr = Literal["dont_wrap", "wrap_long", "wrap_all"]

FIELD_GROWTH_POLICY = bidict(
    fields_stay_at_size=QtWidgets.QFormLayout.FieldsStayAtSizeHint,
    expanding_fields_grow=QtWidgets.QFormLayout.ExpandingFieldsGrow,
    all_non_fixed_fields_grow=QtWidgets.QFormLayout.AllNonFixedFieldsGrow,
)

FieldGrowthPolicyStr = Literal[
    "fields_stay_at_size", "expanding_fields_grow", "all_non_fixed_fields_grow"
]

QtWidgets.QFormLayout.__bases__ = (widgets.Layout,)


class FormLayout(QtWidgets.QFormLayout):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.set_size_mode("maximum")
        self.setVerticalSpacing(8)

    def __setitem__(
        self, index: Union[int, Tuple[int, RoleStr]], value: Union[str, QtWidgets.QWidget]
    ):
        if isinstance(index, tuple):
            row = index[0]
            role = index[1]
        else:
            row = index
            role = "both"
        self.set_widget(value, row, role)

    def __delitem__(self, index: int):
        self.removeRow(index)

    def __iter__(self):
        return iter(self[i] for i in range(self.count()) if self[i] is not None)

    def __len__(self) -> int:
        """Needed for PySide2."""
        return self.rowCount()

    def __add__(self, other):
        if isinstance(other, (QtWidgets.QWidget, QtWidgets.QLayout, tuple)):
            self.add(other)
            return self
        raise TypeError("Wrong type for addition")

    def serialize_fields(self):
        widget_list = []
        positions = []
        for i, item in enumerate(list(self)):
            widget_list.append(item)
            positions.append(self.get_item_position(i))
        return dict(widgets=widget_list, positions=positions)

    def __reduce__(self):
        return type(self), (), self.__getstate__()

    def __setstate__(self, state):
        for i, (item, pos) in enumerate(zip(state["widgets"], state["positions"])):
            self.set_widget(item, pos[0], pos[1])

    def set_widget(
        self, widget: Union[str, QtWidgets.QWidget], row: int, role: RoleStr = "both"
    ):
        if isinstance(widget, str):
            widget = widgets.Label(widget)
        self.setWidget(row, ROLE[role], widget)

    def get_widget(
        self, row: int, role: RoleStr = "both"
    ) -> Union[QtWidgets.QLayout, QtWidgets.QWidget]:
        item = self.itemAt(row, ROLE[role])
        widget = item.widget()
        if widget is None:
            widget = item.layout()
        return widget

    def get_item_position(self, index: int) -> Optional[Tuple[int, RoleStr]]:
        pos = self.getItemPosition(index)
        if pos[0] == -1:
            return None
        return pos[0], ROLE.inverse[pos[1]]

    @classmethod
    def build_from_dict(
        cls,
        dct: Dict[QtWidgets.QWidget, QtWidgets.QWidget],
        parent: Optional[QtWidgets.QWidget] = None,
    ) -> FormLayout:
        formlayout = cls(parent)
        for i, (k, v) in enumerate(dct.items(), start=1):
            if k is not None:
                formlayout.set_widget(k, i, "left")
            if v is not None:
                formlayout.set_widget(v, i, "right")
        return formlayout

    def add(self, *items):
        for i in items:
            if isinstance(i, (QtWidgets.QWidget, QtWidgets.QLayout)):
                self.addRow(i)
            if isinstance(i, tuple):
                self.addRow(*i)

    def set_row_wrap_policy(self, policy: RowWrapPolicyStr):
        """Set row wrap policy to use.

        Args:
            policy: row wrap policy to use

        Raises:
            InvalidParamError: row wrap policy does not exist
        """
        if policy not in ROW_WRAP_POLICY:
            raise InvalidParamError(policy, ROW_WRAP_POLICY)
        self.setRowWrapPolicy(ROW_WRAP_POLICY[policy])

    def get_row_wrap_policy(self) -> RowWrapPolicyStr:
        """Return current row wrap policy.

        Returns:
            row wrap policy
        """
        return ROW_WRAP_POLICY.inverse[self.rowWrapPolicy()]

    def set_field_growth_policy(self, policy: FieldGrowthPolicyStr):
        """Set field growth policy to use.

        Args:
            policy: field growth policy to use

        Raises:
            InvalidParamError: field growth policy does not exist
        """
        if policy not in FIELD_GROWTH_POLICY:
            raise InvalidParamError(policy, FIELD_GROWTH_POLICY)
        self.setFieldGrowthPolicy(FIELD_GROWTH_POLICY[policy])

    def get_field_growth_policy(self) -> FieldGrowthPolicyStr:
        """Return current field growth policy.

        Returns:
            field growth policy
        """
        return FIELD_GROWTH_POLICY.inverse[self.fieldGrowthPolicy()]


if __name__ == "__main__":
    app = widgets.app()
    dct = {"key": widgets.Label("test"), None: widgets.Label("test 2")}
    layout = FormLayout.build_from_dict(dct)
    layout[3] = "hellooo"
    w = widgets.Widget()
    w.set_layout(layout)
    w.show()
    app.main_loop()



from typing import Union

from qtpy import QtNetwork

from prettyqt import core, network
from prettyqt.utils import bidict, InvalidParamError


BIND_MODE = bidict(
    share_address=QtNetwork.QAbstractSocket.ShareAddress,
    dont_share_address=QtNetwork.QAbstractSocket.DontShareAddress,
    reuse_address_hint=QtNetwork.QAbstractSocket.ReuseAddressHint,
    default_for_platform=QtNetwork.QAbstractSocket.DefaultForPlatform,
)

NETWORK_LAYER_PROTOCOL = bidict(
    ipv4=QtNetwork.QAbstractSocket.IPv4Protocol,
    ipv6=QtNetwork.QAbstractSocket.IPv6Protocol,
    any_ip=QtNetwork.QAbstractSocket.AnyIPProtocol,
    unknown=QtNetwork.QAbstractSocket.UnknownNetworkLayerProtocol,
)

PAUSE_MODE = bidict(
    never=QtNetwork.QAbstractSocket.PauseNever,
    on_ssl_errors=QtNetwork.QAbstractSocket.PauseOnSslErrors,
)
QAbstractSocket = QtNetwork.QAbstractSocket

SOCKET_ERROR = bidict(
    connection_refused=QAbstractSocket.ConnectionRefusedError,
    remote_host_closed=QAbstractSocket.RemoteHostClosedError,
    host_not_found=QAbstractSocket.HostNotFoundError,
    socket_access=QAbstractSocket.SocketAccessError,
    socket_resource=QAbstractSocket.SocketResourceError,
    socket_timeout=QAbstractSocket.SocketTimeoutError,
    diagram_too_large=QAbstractSocket.DatagramTooLargeError,
    network=QAbstractSocket.NetworkError,
    address_in_use=QAbstractSocket.AddressInUseError,
    socket_address_not_available=QAbstractSocket.SocketAddressNotAvailableError,
    unsupported_socket_operation=QAbstractSocket.UnsupportedSocketOperationError,
    proxy_authentication_required=QAbstractSocket.ProxyAuthenticationRequiredError,
    ssl_handshake_failed=QAbstractSocket.SslHandshakeFailedError,
    unfinished_socket_operation=QAbstractSocket.UnfinishedSocketOperationError,
    proxy_connection_refused=QAbstractSocket.ProxyConnectionRefusedError,
    proxy_connection_closed=QAbstractSocket.ProxyConnectionClosedError,
    proxy_connection_timeout=QAbstractSocket.ProxyConnectionTimeoutError,
    proxy_not_found=QAbstractSocket.ProxyNotFoundError,
    proxy_protocol=QAbstractSocket.ProxyProtocolError,
    operation=QAbstractSocket.OperationError,
    ssl_internal=QAbstractSocket.SslInternalError,
    ssl_invalid_user_data=QAbstractSocket.SslInvalidUserDataError,
    temporary=QAbstractSocket.TemporaryError,
    unknown_socket=QAbstractSocket.UnknownSocketError,
)

SOCKET_OPTION = bidict(
    low_delay=QtNetwork.QAbstractSocket.LowDelayOption,
    keep_alive=QtNetwork.QAbstractSocket.KeepAliveOption,
    multicast_ttl=QtNetwork.QAbstractSocket.MulticastTtlOption,
    multicast_loopback=QtNetwork.QAbstractSocket.MulticastLoopbackOption,
    type_of_service=QtNetwork.QAbstractSocket.TypeOfServiceOption,
    send_buffer_size_socket=QtNetwork.QAbstractSocket.SendBufferSizeSocketOption,
    receive_buffer_size=QtNetwork.QAbstractSocket.ReceiveBufferSizeSocketOption,
    path_mtu_socket=QtNetwork.QAbstractSocket.PathMtuSocketOption,
)


SOCKET_STATE = bidict(
    unconnected=QtNetwork.QAbstractSocket.UnconnectedState,
    host_lookup=QtNetwork.QAbstractSocket.HostLookupState,
    connecting=QtNetwork.QAbstractSocket.ConnectingState,
    connected=QtNetwork.QAbstractSocket.ConnectedState,
    bound=QtNetwork.QAbstractSocket.BoundState,
    closing=QtNetwork.QAbstractSocket.ClosingState,
    listening=QtNetwork.QAbstractSocket.ListeningState,
)

SOCKET_TYPE = bidict(
    tcp=QtNetwork.QAbstractSocket.TcpSocket,
    udp=QtNetwork.QAbstractSocket.UdpSocket,
    sctp=QtNetwork.QAbstractSocket.SctpSocket,
    unknown=QtNetwork.QAbstractSocket.UnknownSocketType,
)

TYPE_OF_SERVICE = bidict(
    network_control=224,
    internetwork_control=192,
    critic_ecp=160,
    flash_override=128,
    flash=96,
    immediate=64,
    priority=32,
    routine=0,
)

QtNetwork.QAbstractSocket.__bases__ = (core.IODevice,)


class AbstractSocket(QtNetwork.QAbstractSocket):
    def __repr__(self):
        return f"{type(self).__name__}()"

    def bind(
        self,
        address: Union[str, QtNetwork.QHostAddress],
        port: int = 0,
        bind_mode: Union[int, str] = "default_for_platform",
    ):
        if isinstance(address, str):
            address = QtNetwork.QHostAddress(address)
        if bind_mode in BIND_MODE:
            bind_mode = BIND_MODE[bind_mode]
        super().bind(address, port, bind_mode)

    def connect_to_host(
        self,
        hostname: str,
        port: int,
        open_mode: Union[int, str] = "read_write",
        protocol: Union[int, str] = "any_ip",
    ):
        if open_mode in core.iodevice.OPEN_MODES:
            open_mode = core.iodevice.OPEN_MODES[open_mode]
        if protocol in NETWORK_LAYER_PROTOCOL:
            protocol = NETWORK_LAYER_PROTOCOL[protocol]
        super().connectToHost(hostname, port, open_mode, protocol)

    def get_error(self) -> str:
        return SOCKET_ERROR.inverse[self.error()]

    def set_pause_mode(self, mode: str):
        """Set pause mode.

        Valid values: "never", "on_ssl_errors"

        Args:
            mode: pause mode

        Raises:
            InvalidParamError: pause mode does not exist
        """
        if mode not in PAUSE_MODE:
            raise InvalidParamError(mode, PAUSE_MODE)
        self.setPauseMode(PAUSE_MODE[mode])

    def get_pause_mode(self) -> str:
        return PAUSE_MODE.inverse[self.pauseMode()]

    def get_proxy(self) -> network.NetworkProxy:
        return network.NetworkProxy(self.proxy())

    # def set_socket_option(self, name: str, value):
    #     if name not in SOCKET_OPTION:
    #         raise InvalidParamError(name, SOCKET_OPTION)
    #     self.setSocketOption(SOCKET_OPTION[name], value)

    # def get_socket_option(self, name: str):
    #     return self.socketOption(SOCKET_OPTION[name])

    # def set_type_of_service(self, typ: str):
    #     if typ not in TYPE_OF_SERVICE:
    #         raise InvalidParamError(typ, TYPE_OF_SERVICE)
    #     self.set_socket_option("type_of_service", TYPE_OF_SERVICE[typ])

    # def get_type_of_service(self):
    #     opt = self.get_socket_option("type_of_service")
    #     return TYPE_OF_SERVICE.inverse[opt]

    def get_socket_type(self) -> str:
        return SOCKET_TYPE.inverse[self.socketType()]

    def get_state(self) -> str:
        return SOCKET_STATE.inverse[self.state()]

    def get_local_address(self) -> network.HostAddress:
        return network.HostAddress(self.localAddress())



from qtpy import QtMultimedia

from prettyqt import core, multimedia
from prettyqt.utils import bidict, InvalidParamError


EXPOSURE_MODES = bidict(
    auto=QtMultimedia.QCameraExposure.ExposureAuto,
    manual=QtMultimedia.QCameraExposure.ExposureManual,
    portrait=QtMultimedia.QCameraExposure.ExposurePortrait,
    night=QtMultimedia.QCameraExposure.ExposureNight,
    backlight=QtMultimedia.QCameraExposure.ExposureBacklight,
    spotlight=QtMultimedia.QCameraExposure.ExposureSpotlight,
    sports=QtMultimedia.QCameraExposure.ExposureSports,
    snow=QtMultimedia.QCameraExposure.ExposureSnow,
    beach=QtMultimedia.QCameraExposure.ExposureBeach,
    large_aperture=QtMultimedia.QCameraExposure.ExposureLargeAperture,
    small_aperture=QtMultimedia.QCameraExposure.ExposureSmallAperture,
    action=QtMultimedia.QCameraExposure.ExposureAction,
    landscape=QtMultimedia.QCameraExposure.ExposureLandscape,
    night_portrait=QtMultimedia.QCameraExposure.ExposureNightPortrait,
    theatre=QtMultimedia.QCameraExposure.ExposureTheatre,
    sunset=QtMultimedia.QCameraExposure.ExposureSunset,
    steady_photo=QtMultimedia.QCameraExposure.ExposureSteadyPhoto,
    fireworks=QtMultimedia.QCameraExposure.ExposureFireworks,
    party=QtMultimedia.QCameraExposure.ExposureParty,
    candlelight=QtMultimedia.QCameraExposure.ExposureCandlelight,
    barcode=QtMultimedia.QCameraExposure.ExposureBarcode,
    mode_vendor=QtMultimedia.QCameraExposure.ExposureModeVendor,
)

FLASH_MODES = bidict(
    auto=QtMultimedia.QCameraExposure.FlashAuto,
    flash_off=QtMultimedia.QCameraExposure.FlashOff,
    flash_on=QtMultimedia.QCameraExposure.FlashOn,
    red_eye_reduction=QtMultimedia.QCameraExposure.FlashRedEyeReduction,
    fill=QtMultimedia.QCameraExposure.FlashFill,
    torch=QtMultimedia.QCameraExposure.FlashTorch,
    video_light=QtMultimedia.QCameraExposure.FlashVideoLight,
    sync_front_curtain=QtMultimedia.QCameraExposure.FlashSlowSyncFrontCurtain,
    sync_rear_curtain=QtMultimedia.QCameraExposure.FlashSlowSyncRearCurtain,
    manual=QtMultimedia.QCameraExposure.FlashManual,
)

METERING_MODES = bidict(
    matrix=QtMultimedia.QCameraExposure.MeteringMatrix,
    average=QtMultimedia.QCameraExposure.MeteringAverage,
    spot=QtMultimedia.QCameraExposure.MeteringSpot,
)

QtMultimedia.QCameraExposure.__bases__ = (core.Object,)


class CameraExposure(core.Object):
    def __init__(self, item: QtMultimedia.QCameraExposure):
        self.item = item

    def __getattr__(self, val):
        return getattr(self.item, val)

    def set_exposure_mode(self, mode: str):
        """Set the exposure mode.

        Allowed values are "auto", "manual", "portrait", "night", "backlight",
                           "spotlight", "sports", "snow", "beach", "large_aperture",
                           "small_aperture", "action", "landscape", "night_portrait",
                           "theatre", "sunset", "steady_photo", "fireworks", "party",
                           "candlelight", "barcode", "mode_vendor"

        Args:
            mode: exposure mode

        Raises:
            InvalidParamError: exposure mode does not exist
        """
        if mode not in EXPOSURE_MODES:
            raise InvalidParamError(mode, EXPOSURE_MODES)
        self.item.setExposureMode(EXPOSURE_MODES[mode])

    def get_exposure_mode(self) -> str:
        """Return current exposure mode.

        Possible values: "auto", "manual", "portrait", "night", "backlight",
                         "spotlight", "sports", "snow", "beach", "large_aperture",
                         "small_aperture", "action", "landscape", "night_portrait",
                         "theatre", "sunset", "steady_photo", "fireworks", "party",
                         "candlelight", "barcode", "mode_vendor"

        Returns:
            exposure mode
        """
        return EXPOSURE_MODES.inverse[self.item.exposureMode()]

    def set_flash_mode(self, mode: str):
        """Set the flash mode.

        Allowed values are "auto", "flash_on", "flash_off", "red_eye_reduction",
                           "fill", "torch", "video_light", "sync_front_curtain",
                           "sync_rear_curtain", "manual"

        Args:
            mode: flash mode

        Raises:
            InvalidParamError: flash mode does not exist
        """
        if mode not in FLASH_MODES:
            raise InvalidParamError(mode, FLASH_MODES)
        self.item.setFlashMode(FLASH_MODES[mode])

    def get_flash_mode(self) -> str:
        """Return current flash mode.

        Possible values: "auto", "flash_on", "flash_off", "red_eye_reduction",
                         "fill", "torch", "video_light", "sync_front_curtain",
                         "sync_rear_curtain", "manual"

        Returns:
            flash mode
        """
        return FLASH_MODES.inverse[self.item.flashMode()]

    def set_metering_mode(self, mode: str):
        """Set the metering mode.

        Allowed values are "matrix", "average", "spot"

        Args:
            mode: metering mode

        Raises:
            InvalidParamError: metering mode does not exist
        """
        if mode not in METERING_MODES:
            raise InvalidParamError(mode, METERING_MODES)
        self.item.setMeteringMode(METERING_MODES[mode])

    def get_metering_mode(self) -> str:
        """Return current metering mode.

        Possible values: "matrix", "average", "spot"

        Returns:
            metering mode
        """
        return METERING_MODES.inverse[self.item.meteringMode()]

    def get_spot_metering_point(self) -> core.PointF:
        return core.PointF(self.item.spotMeteringPoint())

    def is_exposure_mode_supported(self, mode: str) -> bool:
        if mode not in EXPOSURE_MODES:
            raise InvalidParamError(mode, EXPOSURE_MODES)
        return self.item.isExposureModeSupported(EXPOSURE_MODES[mode])

    def is_flash_mode_supported(self, mode: str) -> bool:
        if mode not in FLASH_MODES:
            raise InvalidParamError(mode, FLASH_MODES)
        return self.item.isFlashModeSupported(FLASH_MODES[mode])

    def is_metering_mode_supported(self, mode: str) -> bool:
        if mode not in METERING_MODES:
            raise InvalidParamError(mode, METERING_MODES)
        return self.item.isMeteringModeSupported(METERING_MODES[mode])


if __name__ == "__main__":
    cam = multimedia.Camera()
    focus = cam.get_focus()



from qtpy import QtGui, QtCore, QtWidgets

from prettyqt import widgets, gui, core, constants

DecorationRole2 = QtCore.Qt.UserRole + 1000


class IconDelegate(widgets.StyledItemDelegate):
    def paint(
        self,
        painter: QtGui.QPainter,
        option: QtWidgets.QStyleOptionViewItem,
        index: QtCore.QModelIndex,
    ):
        """Override to paint an icon based on given Pixmap / Color / Icon.

        Pixmap / Color / Icon must be set to 'QtCore.Qt.UserRole + 1000'

        Args:
            painter (QtGui.QPainter): painter to paint the icon
            option (QtWidgets.QStyleOptionViewItem): state of the item to be displayed
            index (QtCore.QModelIndex): index which gets decorated
        """
        super().paint(painter, option, index)
        value = index.data(DecorationRole2)
        if value:
            margin = 10
            mode = gui.Icon.Normal

            if not (option.state & widgets.Style.State_Enabled):
                mode = gui.Icon.Disabled
            elif option.state & widgets.Style.State_Selected:
                mode = gui.Icon.Selected

            if isinstance(value, QtGui.QPixmap):
                icon = QtGui.QIcon(value)
                option.decorationSize = value.size() / value.devicePixelRatio()

            elif isinstance(value, QtGui.QColor):
                pixmap = QtGui.QPixmap(option.decorationSize)
                pixmap.fill(value)
                icon = QtGui.QIcon(pixmap)

            elif isinstance(value, QtGui.QImage):
                icon = QtGui.QIcon(QtGui.QPixmap.fromImage(value))
                option.decorationSize = value.size() / value.devicePixelRatio()

            elif isinstance(value, QtGui.QIcon):
                state = (
                    gui.Icon.On
                    if option.state & widgets.Style.State_Open
                    else gui.Icon.Off
                )
                actualSize = option.icon.actualSize(option.decorationSize, mode, state)
                option.decorationSize = core.Size(
                    min(option.decorationSize.width(), actualSize.width()),
                    min(option.decorationSize.height(), actualSize.height()),
                )

            r = core.Rect(core.Point(), option.decorationSize)
            r.moveCenter(option.rect.center())
            r.setRight(option.rect.right() - margin)
            state = (
                gui.Icon.On if option.state & widgets.Style.State_Open else gui.Icon.Off
            )
            icon.paint(
                painter, r, constants.ALIGN_RIGHT | constants.ALIGN_V_CENTER, mode, state
            )



# from qtpy import QtWebEngineWidgets

from typing import Literal

from qtpy import PYQT5, PYSIDE2

if PYQT5:
    from PyQt5 import QtWebEngineWidgets  # type: ignore
elif PYSIDE2:
    from PySide2 import QtWebEngineWidgets

from prettyqt import core, webenginewidgets
from prettyqt.utils import bidict, InvalidParamError


HTTP_CACHE_TYPE = bidict(
    none=QtWebEngineWidgets.QWebEngineProfile.NoCache,
    disk=QtWebEngineWidgets.QWebEngineProfile.DiskHttpCache,
    memory=QtWebEngineWidgets.QWebEngineProfile.MemoryHttpCache,
)

HttpCacheTypeStr = Literal["none", "disk", "memory"]

PERSISTENT_COOKIE_POLICY = bidict(
    none=QtWebEngineWidgets.QWebEngineProfile.NoPersistentCookies,
    allow=QtWebEngineWidgets.QWebEngineProfile.AllowPersistentCookies,
    force=QtWebEngineWidgets.QWebEngineProfile.ForcePersistentCookies,
)

PersistentCookiePolicyStr = Literal["none", "allow", "force"]

QtWebEngineWidgets.QWebEngineProfile.__bases__ = (core.Object,)


class WebEngineProfile(QtWebEngineWidgets.QWebEngineProfile):
    def set_persistent_cookie_policy(self, policy: PersistentCookiePolicyStr):
        """Set the persistent cookie policy.

        Args:
            policy: persistent cookie policy

        Raises:
            InvalidParamError: Policy does not exist
        """
        if policy not in PERSISTENT_COOKIE_POLICY:
            raise InvalidParamError(policy, PERSISTENT_COOKIE_POLICY)
        self.setPersistentCookiesPolicy(PERSISTENT_COOKIE_POLICY[policy])

    def get_persistent_cookie_policy(self) -> PersistentCookiePolicyStr:
        """Return current persistent cookie policy.

        Returns:
            Persistent cookie policy
        """
        return PERSISTENT_COOKIE_POLICY.inverse[self.persistentCookiesPolicy()]

    def set_http_cache_type(self, typ: HttpCacheTypeStr):
        """Set the http cache type.

        Args:
            typ: http cache type

        Raises:
            InvalidParamError: Cache type does not exist
        """
        if typ not in HTTP_CACHE_TYPE:
            raise InvalidParamError(typ, HTTP_CACHE_TYPE)
        self.setHttpCacheType(HTTP_CACHE_TYPE[typ])

    def get_http_cache_type(self) -> HttpCacheTypeStr:
        """Return current http cache type.

        Returns:
            Http cache type
        """
        return HTTP_CACHE_TYPE.inverse[self.httpCacheType()]

    def get_scripts(self) -> webenginewidgets.WebEngineScriptCollection:
        return webenginewidgets.WebEngineScriptCollection(self.scripts())


if __name__ == "__main__":
    from prettyqt import widgets

    app = widgets.app()
    item = WebEngineProfile()
    app.main_loop()



from typing import Union, Literal
import contextlib

from qtpy import QtCore, QtGui

from prettyqt import core, gui
from prettyqt.utils import bidict, colors, InvalidParamError


COMPOSITION_MODE = bidict(
    source_over=QtGui.QPainter.CompositionMode_SourceOver,
    destination_over=QtGui.QPainter.CompositionMode_DestinationOver,
    clear=QtGui.QPainter.CompositionMode_Clear,
    source=QtGui.QPainter.CompositionMode_Source,
    destination=QtGui.QPainter.CompositionMode_Destination,
    source_in=QtGui.QPainter.CompositionMode_SourceIn,
    destination_in=QtGui.QPainter.CompositionMode_DestinationIn,
    source_out=QtGui.QPainter.CompositionMode_SourceOut,
    destination_out=QtGui.QPainter.CompositionMode_DestinationOut,
    source_atop=QtGui.QPainter.CompositionMode_SourceAtop,
    destination_atop=QtGui.QPainter.CompositionMode_DestinationAtop,
)

CompositionModeStr = Literal[
    "source_over",
    "destination_over",
    "clear",
    "source",
    "destination",
    "source_in",
    "destination_in",
    "source_out",
    "destination_out",
    "source_atop",
    "destination_atop",
]

PATTERN = bidict(
    none=QtCore.Qt.NoBrush,
    solid=QtCore.Qt.SolidPattern,
    dense_1=QtCore.Qt.Dense1Pattern,
    dense_2=QtCore.Qt.Dense2Pattern,
    dense_3=QtCore.Qt.Dense3Pattern,
    dense_4=QtCore.Qt.Dense4Pattern,
    dense_5=QtCore.Qt.Dense5Pattern,
    dense_6=QtCore.Qt.Dense6Pattern,
    dense_7=QtCore.Qt.Dense7Pattern,
    horizontal=QtCore.Qt.HorPattern,
    vertical=QtCore.Qt.VerPattern,
    cross=QtCore.Qt.CrossPattern,
    backward_diagonal=QtCore.Qt.BDiagPattern,
    forward_diagonal=QtCore.Qt.FDiagPattern,
    crossing_diagonal=QtCore.Qt.DiagCrossPattern,
    linear_gradient=QtCore.Qt.LinearGradientPattern,
    conical_gradient=QtCore.Qt.ConicalGradientPattern,
    radial_gradient=QtCore.Qt.RadialGradientPattern,
    texture=QtCore.Qt.TexturePattern,
)

PatternStr = Literal[
    "none",
    "solid",
    "dense_1",
    "dense_2",
    "dense_3",
    "dense_4",
    "dense_5",
    "dense_6",
    "dense_7",
    "horizontal",
    "vertical",
    "cross",
    "backward_diagonal",
    "forward_diagonal",
    "crossing_diagonal",
    "linear_gradient",
    "conical_gradient",
    "radial_gradient",
    "texture",
]

CLIP_OPERATION = bidict(
    none=QtCore.Qt.NoClip,
    replace=QtCore.Qt.ReplaceClip,
    intersect=QtCore.Qt.IntersectClip,
)

ClipOperationStr = Literal["none", "replace", "intersect"]

RENDER_HINTS = bidict(
    antialiasing=QtGui.QPainter.Antialiasing,
    text_antialiasing=QtGui.QPainter.TextAntialiasing,
    smooth_pixmap_transform=QtGui.QPainter.SmoothPixmapTransform,
    high_quality_antialiasing=QtGui.QPainter.HighQualityAntialiasing,
    noncosmetic_default_pen=QtGui.QPainter.NonCosmeticDefaultPen,
    qt4_compatible_painting=QtGui.QPainter.Qt4CompatiblePainting,
    lossless_image_rendering=QtGui.QPainter.LosslessImageRendering,
)

FILL_RULE = bidict(odd_even=QtCore.Qt.OddEvenFill, winding=QtCore.Qt.WindingFill)

FillRuleStr = Literal["odd_even", "winding"]


class Painter(QtGui.QPainter):
    def __enter__(self):
        return self

    def __exit__(self, typ, value, traceback):
        self.end()

    @contextlib.contextmanager
    def paint_on(self, obj):
        self.begin(obj)
        yield self
        self.end()

    @contextlib.contextmanager
    def backup_state(self):
        self.save()
        yield self
        self.restore()

    def draw_image(
        self,
        target: Union[QtCore.QPoint, QtCore.QPointF, QtCore.QRect, QtCore.QRectF],
        frame_buffer: QtGui.QImage,
    ):
        self.set_composition_mode("source_atop")
        self.drawImage(target, frame_buffer)

    def draw_polygon(self, *args, fill_rule: FillRuleStr = "odd_even"):
        if fill_rule not in FILL_RULE:
            raise InvalidParamError(fill_rule, FILL_RULE)
        self.drawPolygon(*args, fillRule=FILL_RULE[fill_rule])

    def use_antialiasing(self):
        self.setRenderHint(self.Antialiasing, True)

    def fill_rect(
        self,
        rect: Union[QtCore.QRectF, QtCore.QRect],
        color,
        pattern: PatternStr = "solid",
    ):
        if pattern not in PATTERN:
            raise InvalidParamError(pattern, PATTERN)
        if isinstance(rect, tuple):
            rect = core.Rect(*rect)
        if isinstance(color, str):
            if color not in gui.Color.colorNames():
                raise ValueError("Invalid value for color.")
            color = gui.Color(color)
        if pattern != "solid":
            color = gui.Brush(color, PATTERN[pattern])
        self.fillRect(rect, color)

    def set_pen(
        self,
        style: PatternStr = "solid",
        width: float = 1.0,
        color: colors.ColorType = "black",
        join_style: gui.pen.JoinStyleStr = "bevel",
        cap_style: gui.pen.CapStyleStr = "square",
    ):
        """Set pen to use.

        Args:
            style: pen style to use
            width: pen width
            color: pen color
            join_style: pen join style to use
            cap_style: pen cap style to use
        """
        pen = gui.Pen()
        pen.set_style(style)
        pen.set_cap_style(cap_style)
        pen.set_join_style(join_style)
        pen.setWidthF(width)
        pen.set_color(color)
        self.setPen(pen)

    def get_pen(self) -> gui.Pen:
        """Return current pen.

        Returns:
            current pen
        """
        return gui.Pen(self.pen())

    def set_color(self, color: colors.ColorType):
        color = colors.get_color(color)
        self.setPen(color)

    def set_brush(self, brush: Union[QtGui.QBrush, colors.ColorType]):
        if not isinstance(brush, QtGui.QBrush):
            brush = colors.get_color(brush)
        self.setBrush(brush)

    def set_transparent_background(self, transparent: bool = True):
        mode = QtCore.Qt.TransparentMode if transparent else QtCore.Qt.OpaqueMode
        self.setBackgroundMode(mode)

    def set_composition_mode(self, mode: CompositionModeStr):
        """Set the current composition mode.

        Raises:
            InvalidParamError: composition mode does not exist
        """
        if mode not in COMPOSITION_MODE:
            raise InvalidParamError(mode, COMPOSITION_MODE)
        self.setCompositionMode(COMPOSITION_MODE[mode])

    def get_composition_mode(self) -> CompositionModeStr:
        """Get the current composition mode.

        Returns:
            composition mode
        """
        return COMPOSITION_MODE.inverse[self.compositionMode()]

    def set_clip_path(
        self, path: QtGui.QPainterPath, operation: ClipOperationStr = "replace"
    ):
        if operation not in CLIP_OPERATION:
            raise InvalidParamError(operation, CLIP_OPERATION)
        self.setClipPath(path, CLIP_OPERATION[operation])

    def get_text_rect(self, text: str) -> core.Rect:
        return self.drawText(core.Rect(), QtCore.Qt.TextDontPrint, text)

    @contextlib.contextmanager
    def clip_path(self, operation: ClipOperationStr = "replace"):
        path = gui.PainterPath()
        yield path
        self.set_clip_path(path, operation)



import logging
from typing import Union, Literal
import pathlib

from qtpy import QtGui

from prettyqt.utils import bidict, InvalidParamError

logger = logging.getLogger(__name__)

SYSTEM_FONT = bidict(
    general=QtGui.QFontDatabase.GeneralFont,
    fixed=QtGui.QFontDatabase.FixedFont,
    title=QtGui.QFontDatabase.TitleFont,
    smallest_readable=QtGui.QFontDatabase.SmallestReadableFont,
)

SystemFontStr = Literal["general", "fixed", "title", "smallest_readable"]


class FontDatabase(QtGui.QFontDatabase):
    @classmethod
    def add_fonts_from_folder(cls, path: Union[str, pathlib.Path]):
        if isinstance(path, str):
            path = pathlib.Path(path)
        for p in path.iterdir():
            if p.suffix.lower() in [".ttf", ".otf"]:
                logger.debug(f"adding font {p} to database.")
                cls.addApplicationFont(str(p))

    def get_system_font(self, font_type: SystemFontStr):
        if font_type not in SYSTEM_FONT:
            raise InvalidParamError(font_type, SYSTEM_FONT)
        return self.systemFont(SYSTEM_FONT[font_type])


if __name__ == "__main__":
    from prettyqt import gui

    app = gui.app()
    db = FontDatabase()
    print(db.families())



from qtpy import QtWidgets

from prettyqt import widgets, gui


QtWidgets.QGraphicsOpacityEffect.__bases__ = (widgets.GraphicsEffect,)


class GraphicsOpacityEffect(QtWidgets.QGraphicsOpacityEffect):
    def serialize_fields(self):
        return dict(opacity=self.opacity(), opacityMask=gui.Brush(self.opacityMask()))



from qtpy import QtWidgets

from prettyqt import widgets


QtWidgets.QStyleOptionDockWidget.__bases__ = (widgets.StyleOption,)


class StyleOptionDockWidget(QtWidgets.QStyleOptionDockWidget):
    pass



from typing import Dict, Any, Literal

from qtpy import QtCore, QtGui

from prettyqt import core
from prettyqt.utils import bidict, InvalidParamError


SHAPE = bidict(
    arrow=QtCore.Qt.ArrowCursor,
    up_arrow=QtCore.Qt.UpArrowCursor,
    cross=QtCore.Qt.CrossCursor,
    wait=QtCore.Qt.WaitCursor,
    caret=QtCore.Qt.IBeamCursor,
)

ShapeStr = Literal["arrow", "up_arrow", "cross", "wait", "caret"]


class Cursor(QtGui.QCursor):
    def __getstate__(self):
        return bytes(self)

    def __setstate__(self, ba):
        super().__init__()
        core.DataStream.write_bytearray(ba, self)

    def __bytes__(self):
        ba = core.DataStream.create_bytearray(self)
        return bytes(ba)

    def serialize_fields(self) -> Dict[str, Any]:
        return dict(shape=self.get_shape())

    def set_shape(self, shape: ShapeStr):
        """Set cursor shape.

        Args:
            shape: shape to use

        Raises:
            InvalidParamError: shape does not exist
        """
        if shape not in SHAPE:
            raise InvalidParamError(shape, SHAPE)
        self.setShape(SHAPE[shape])

    def get_shape(self) -> ShapeStr:
        """Return current cursor shape.

        Returns:
            cursor shape
        """
        return SHAPE.inverse[self.shape()]



import sys
from typing import Optional

from qtpy import QtWidgets

from prettyqt import constants, gui, core, multimedia, widgets, multimediawidgets


class Player(widgets.MainWindow):
    def __init__(self, parent: Optional[QtWidgets.QWidget] = None):
        super().__init__(parent)
        self.playback_rate = 1.0
        self.playlist = multimedia.MediaPlaylist(self)
        self.player = multimedia.MediaPlayer(self)

        toolbar = widgets.ToolBar()
        self.addToolBar(toolbar)

        file_menu = self.menuBar().add_menu("&File")
        open_action = widgets.Action(
            icon=gui.Icon.fromTheme("document-open"),
            text="&Open...",
            parent=self,
            shortcut=gui.KeySequence.Open,
            callback=self.open,
        )
        exit_action = widgets.Action(
            icon=gui.Icon.fromTheme("application-exit"),
            text="E&xit",
            parent=self,
            shortcut="Ctrl+Q",
            callback=self.close,
        )

        play_menu = self.menuBar().add_menu("&Play")
        self.play_action = toolbar.add_action(
            icon=widgets.Application.get_style_icon("media_play"),
            label="Play",
            callback=self.player.play,
        )
        self.previous_action = toolbar.add_action(
            icon=widgets.Application.get_style_icon("media_skip_backward"),
            label="Previous",
            callback=self.previous_clicked,
        )
        self.pause_action = toolbar.add_action(
            icon=widgets.Application.get_style_icon("media_pause"),
            label="Pause",
            callback=self.player.pause,
        )
        self.next_action = toolbar.add_action(
            icon=widgets.Application.get_style_icon("media_skip_backward"),
            label="Next",
            callback=self.playlist.next,
        )

        self.stop_action = toolbar.add_action(
            icon=widgets.Application.get_style_icon("media_stop"),
            label="Stop",
            callback=self.player.stop,
        )
        file_menu.add(open_action)
        file_menu.add(exit_action)

        play_menu.add(self.play_action)
        play_menu.add(self.previous_action)
        play_menu.add(self.pause_action)
        play_menu.add(self.next_action)
        play_menu.add(self.stop_action)

        self.clock = widgets.Label(self)
        self.clock.setText("00:00/00:00")
        # self.clock.setGeometry(550, 660, 80, 30)

        self.vol_slider = widgets.Slider()
        self.vol_slider.set_orientation("horizontal")
        self.vol_slider.set_range(0, 100)
        self.vol_slider.setFixedWidth(120)
        self.vol_slider.set_value(self.player.volume())
        self.vol_slider.setTickInterval(10)
        self.vol_slider.set_tick_position("below")
        self.vol_slider.setToolTip("Volume")
        self.vol_slider.value_changed.connect(self.player.setVolume)

        self.slider = widgets.Slider(constants.HORIZONTAL, self)
        # self.slider.setGeometry(10, 640, 800 - 20, 20)
        self.slider.setRange(0, 100)
        self.slider.value_changed.connect(self.on_slider_change)

        toolbar.add_separator()
        toolbar.addWidget(self.vol_slider)
        toolbar.add_separator()
        toolbar.addWidget(self.clock)
        toolbar.add_separator()
        toolbar.addWidget(self.slider)

        self.video_widget = multimediawidgets.VideoWidget()
        self.setCentralWidget(self.video_widget)
        self.player.setPlaylist(self.playlist)
        self.player.stateChanged.connect(self._update_buttons)
        self.player.setVideoOutput(self.video_widget)
        self.player.positionChanged.connect(self.on_player_change)
        self.player.durationChanged.connect(self.set_media_time)

        self._update_buttons(self.player.state())

    def handle_backward(self):
        if self.playback_rate > 0.5:
            self.playback_rate = self.playback_rate - 0.5
            self.player.setPlaybackRate(self.playback_rate)

    def handle_forward(self):
        if self.playback_rate < 1.5:
            self.playback_rate = self.playback_rate + 0.5
            self.player.setPlaybackRate(self.playback_rate)

    def set_media_time(self, time):
        self.slider.set_value(0)
        self.time = self.player.duration() / 1000
        self.slider.setRange(0, int(self.time))

    def on_player_change(self, val: int):
        with self.slider.block_signals():
            self.slider.set_value(int(val / 1000))
        tmp = self.player.position()
        duration = self.player.duration() / 1000
        # print(tmp, val)
        secs = tmp / 1000
        self.clock.setText(
            "%02d:%02d / %02d:%02d" % (secs / 60, secs % 60, duration / 60, duration % 60)
        )

    def on_slider_change(self, val):
        self.player.setPosition(self.slider.get_value() * 1000)

    def open(self):
        file_dialog = widgets.FileDialog(parent=self)
        # supportedMimeTypes = ["video/mp4", "*.*"]
        # file_dialog.setMimeTypeFilters(supportedMimeTypes)
        movies_location = core.StandardPaths.get_writable_location("movies")
        file_dialog.set_directory(movies_location)
        if file_dialog.exec_() == widgets.Dialog.Accepted:
            self.playlist.add_media(file_dialog.selectedFiles()[0])
            self.player.play()

    def previous_clicked(self):
        # Go to previous track if we are within the first 5 seconds of playback
        # Otherwise, seek to the beginning.
        if self.player.position() <= 5000:
            self.playlist.previous()
        else:
            self.player.setPosition(0)

    def _update_buttons(self, state):
        media_count = len(self.playlist)
        self.play_action.setEnabled(
            media_count > 0 and state != multimedia.MediaPlayer.PlayingState
        )
        self.pause_action.setEnabled(state == multimedia.MediaPlayer.PlayingState)
        self.stop_action.setEnabled(state != multimedia.MediaPlayer.StoppedState)
        self.previous_action.setEnabled(self.player.position() > 0)
        self.next_action.setEnabled(media_count > 1)


if __name__ == "__main__":
    app = widgets.app()
    mainWin = Player()
    mainWin.resize(800, 600)
    mainWin.show()
    sys.exit(app.main_loop())



from qtpy import QtLocation

from prettyqt import location
from prettyqt.utils import bidict

TYPE = bidict(
    unknown=QtLocation.QPlaceSearchResult.UnknownSearchResult,
    place=QtLocation.QPlaceSearchResult.PlaceResult,
    proposed_search=QtLocation.QPlaceSearchResult.ProposedSearchResult,
)


class PlaceSearchResult(QtLocation.QPlaceSearchResult):
    def get_icon(self) -> location.PlaceIcon:
        return location.PlaceIcon(self.icon())

    def get_type(self) -> str:
        """Return result type.

        possible values: "unknown" "place", "proposed_search",

        Returns:
            Result type
        """
        return TYPE.inverse[self.type()]



"""quick module.

contains QtQuick-based classes
"""

from .quicktextdocument import QuickTextDocument
from .quickrendercontrol import QuickRenderControl
from .quickimageresponse import QuickImageResponse
from .quickitemgrabresult import QuickItemGrabResult
from .quickview import QuickView
from .quickitem import QuickItem
from .quickpainteditem import QuickPaintedItem
from .quickwindow import QuickWindow


__all__ = [
    "QuickView",
    "QuickItem",
    "QuickWindow",
    "QuickPaintedItem",
    "QuickTextDocument",
    "QuickRenderControl",
    "QuickImageResponse",
    "QuickItemGrabResult",
]



from typing import Literal

from qtpy.QtCharts import QtCharts

from prettyqt import core, gui
from prettyqt.utils import bidict, InvalidParamError


LABEL_POSITION = bidict(
    outside=QtCharts.QPieSlice.LabelOutside,
    inside_horizontal=QtCharts.QPieSlice.LabelInsideHorizontal,
    inside_tangential=QtCharts.QPieSlice.LabelInsideTangential,
    inside_normal=QtCharts.QPieSlice.LabelInsideNormal,
)

LabelPositionStr = Literal[
    "outside", "inside_horizontal", "inside_tangential", "inside_normal"
]

QtCharts.QPieSlice.__bases__ = (core.Object,)


class PieSlice(QtCharts.QPieSlice):
    def __repr__(self):
        return f"{type(self).__name__}({self.label()!r}, {self.value()})"

    def set_label_position(self, position: LabelPositionStr):
        """Set the label position.

        Args:
            position: label position

        Raises:
            InvalidParamError: label position does not exist
        """
        if position not in LABEL_POSITION:
            raise InvalidParamError(position, LABEL_POSITION)
        self.setLabelPosition(LABEL_POSITION[position])

    def get_label_position(self) -> LabelPositionStr:
        """Return current label position.

        Returns:
            label position
        """
        return LABEL_POSITION.inverse[self.labelPosition()]

    def get_label_font(self) -> gui.Font:
        return gui.Font(self.labelFont())

    def get_label_brush(self) -> gui.Brush:
        return gui.Brush(self.labelBrush())

    def get_label_color(self) -> gui.Color:
        return gui.Color(self.labelColor())

    def get_pen(self) -> gui.Pen:
        return gui.Pen(self.pen())

    def get_brush(self) -> gui.Brush:
        return gui.Brush(self.brush())

    def get_color(self) -> gui.Color:
        return gui.Color(self.color())

    def get_border_color(self) -> gui.Color:
        return gui.Color(self.borderColor())


if __name__ == "__main__":
    pieslice = PieSlice("test", 1)
    print(repr(pieslice))



from qtpy import QtCore

from prettyqt import core
from prettyqt.utils import bidict


PROCESS_EVENT_FLAGS = bidict(
    all=QtCore.QEventLoop.AllEvents,
    exclude_user_input=QtCore.QEventLoop.ExcludeUserInputEvents,
    exclude_socket_notifiers=QtCore.QEventLoop.ExcludeSocketNotifiers,
    wait_for_more=QtCore.QEventLoop.WaitForMoreEvents,
)

QtCore.QEventLoop.__bases__ = (core.Object,)


class EventLoop(QtCore.QEventLoop):
    def execute(
        self,
        user_input: bool = True,
        socket_notifiers: bool = True,
        wait_for_more: bool = False,
    ) -> int:
        flag = 0
        if not user_input:
            flag |= 1
        if not socket_notifiers:
            flag |= 2
        if wait_for_more:
            flag |= 4
        return self.exec_(flag)



from typing import Union, Tuple, Optional

from qtpy import QtCore, QtGui

from prettyqt import core, gui
from prettyqt.utils import InvalidParamError


class PdfWriter(QtGui.QPdfWriter):
    def setup(self, size: QtCore.QSize):
        primary = gui.app().primaryScreen()
        dpi = primary.logicalDotsPerInchX()
        self.setResolution(int(dpi))
        self.set_page_margins((0, 0, 0, 0))
        self.setPageSizeMM(core.SizeF(size.width(), size.height()) / dpi * 25.4)

    def set_page_margins(
        self,
        margins: Union[Tuple[float, float, float, float], QtCore.QMarginsF],
        unit: Optional[str] = None,
    ):
        if isinstance(margins, tuple):
            margins = QtCore.QMarginsF(*margins)
        if unit is None:
            return self.setPageMargins(margins)
        else:
            if unit not in gui.pagelayout.UNITS:
                raise InvalidParamError(unit, gui.pagelayout.UNITS)
            return self.setPageMargins(margins, gui.pagelayout.UNITS[unit])


if __name__ == "__main__":
    app = gui.app()
    writer = PdfWriter("")
    writer.setup(QtCore.QSize(1, 1))



from __future__ import annotations

from typing import Union, Literal
import pathlib
import contextlib

from qtpy import QtCore

from prettyqt.utils import bidict


LOCK_ERROR = bidict(
    none=QtCore.QLockFile.NoError,
    lock_failed=QtCore.QLockFile.LockFailedError,
    permission=QtCore.QLockFile.PermissionError,
    unknown=QtCore.QLockFile.UnknownError,
)

LockErrorStr = Literal["none", "lock_failed", "permission", "unknown"]


class LockFile(QtCore.QLockFile):
    def __init__(self, path: Union[str, pathlib.Path]):
        super().__init__(str(path))

    def get_error(self) -> LockErrorStr:
        return LOCK_ERROR.inverse[self.error()]

    @contextlib.contextmanager
    def lock_file(self):
        self.lock()
        yield self
        self.unlock()



from typing import List, Literal

from qtpy import QtWidgets, QtCore

from prettyqt import core, widgets, gui
from prettyqt.utils import InvalidParamError, bidict, helpers


DRAG_MODE = bidict(
    none=QtWidgets.QGraphicsView.NoDrag,
    scroll_hand=QtWidgets.QGraphicsView.ScrollHandDrag,
    rubber_band=QtWidgets.QGraphicsView.RubberBandDrag,
)

DragModeStr = Literal["none", "scroll_hand", "rubber_band"]

CACHE_MODES = bidict(
    none=QtWidgets.QGraphicsView.CacheNone,
    background=QtWidgets.QGraphicsView.CacheBackground,
)

CacheModeStr = Literal["none", "background"]

OPTIMIZATION_FLAGS = bidict(
    dont_clip_painter=QtWidgets.QGraphicsView.DontClipPainter,
    dont_save_painter_state=QtWidgets.QGraphicsView.DontSavePainterState,
    dont_adjust_for_antialiasing=QtWidgets.QGraphicsView.DontAdjustForAntialiasing,
)

VIEWPORT_ANCHOR = bidict(
    none=QtWidgets.QGraphicsView.NoAnchor,
    view_center=QtWidgets.QGraphicsView.AnchorViewCenter,
    under_mouse=QtWidgets.QGraphicsView.AnchorUnderMouse,
)

ViewportAnchorStr = Literal["none", "view_center", "under_mouse"]

VIEWPORT_UPDATE_MODE = bidict(
    full=QtWidgets.QGraphicsView.FullViewportUpdate,
    minimal=QtWidgets.QGraphicsView.MinimalViewportUpdate,
    smart=QtWidgets.QGraphicsView.SmartViewportUpdate,
    bounding_rect=QtWidgets.QGraphicsView.BoundingRectViewportUpdate,
    none=QtWidgets.QGraphicsView.NoViewportUpdate,
)

ViewportUpdateModeStr = Literal["full", "minimal", "smart", "bounding_rect", "none"]

ITEM_SELECTION_MODE = bidict(
    contains_shape=QtCore.Qt.ContainsItemShape,
    intersects_shape=QtCore.Qt.IntersectsItemShape,
    contains_bounding_rect=QtCore.Qt.ContainsItemBoundingRect,
    intersects_bounding_rect=QtCore.Qt.IntersectsItemBoundingRect,
)

ItemSelectionModeStr = Literal[
    "contains_shape",
    "intersects_shape",
    "contains_bounding_rect",
    "intersects_bounding_rect",
]

QtWidgets.QGraphicsView.__bases__ = (widgets.AbstractScrollArea,)


class GraphicsView(QtWidgets.QGraphicsView):
    def serialize_fields(self):
        return dict(
            scene=self.scene(),
            background_brush=gui.Brush(self.backgroundBrush()),
            foreground_brush=gui.Brush(self.foregroundBrush()),
            transformation_anchor=self.get_transformation_anchor(),
            resize_anchor=self.get_resize_anchor(),
            viewport_update_mode=self.get_viewport_update_mode(),
            drag_mode=self.get_drag_mode(),
            rubberband_selection_mode=self.get_rubberband_selection_mode(),
            scene_rect=core.RectF(self.sceneRect()),
            cache_mode=self.get_cache_mode(),
            is_interactive=self.isInteractive(),
        )

    def __getitem__(self, index: int) -> QtWidgets.QGraphicsItem:
        return self.items()[index]

    def invalidate_scene(
        self, rect: QtCore.QRectF, layer: widgets.graphicsscene.SceneLayerStr = "all"
    ):
        if layer not in widgets.graphicsscene.SCENE_LAYER:
            raise InvalidParamError(layer, widgets.graphicsscene.SCENE_LAYER)
        self.invalidateScene(rect, widgets.graphicsscene.SCENE_LAYER[layer])

    def set_transformation_anchor(self, mode: ViewportAnchorStr):
        """Set how the view should position the scene during transformations.

        Args:
            mode: transformation anchor to use

        Raises:
            InvalidParamError: mode does not exist
        """
        if mode not in VIEWPORT_ANCHOR:
            raise InvalidParamError(mode, VIEWPORT_ANCHOR)
        self.setTransformationAnchor(VIEWPORT_ANCHOR[mode])

    def get_transformation_anchor(self) -> ViewportAnchorStr:
        """Return current transformation anchor.

        Returns:
            viewport anchor
        """
        return VIEWPORT_ANCHOR.inverse[self.transformationAnchor()]

    def set_resize_anchor(self, mode: ViewportAnchorStr):
        """Set how the view should position the scene during resizes.

        Args:
            mode: resize anchor to use

        Raises:
            InvalidParamError: mode does not exist
        """
        if mode not in VIEWPORT_ANCHOR:
            raise InvalidParamError(mode, VIEWPORT_ANCHOR)
        self.setResizeAnchor(VIEWPORT_ANCHOR[mode])

    def get_resize_anchor(self) -> ViewportAnchorStr:
        """Return current resize anchor.

        Returns:
            resize anchor
        """
        return VIEWPORT_ANCHOR.inverse[self.resizeAnchor()]

    def set_viewport_update_mode(self, mode: ViewportUpdateModeStr):
        """Set how the viewport should update its contents.

        Args:
            mode: viewport update mode to use

        Raises:
            InvalidParamError: mode does not exist
        """
        if mode not in VIEWPORT_UPDATE_MODE:
            raise InvalidParamError(mode, VIEWPORT_UPDATE_MODE)
        self.setViewportUpdateMode(VIEWPORT_UPDATE_MODE[mode])

    def get_viewport_update_mode(self) -> ViewportUpdateModeStr:
        """Return current viewport update mode.

        Returns:
            viewport update mode
        """
        return VIEWPORT_UPDATE_MODE.inverse[self.viewportUpdateMode()]

    def set_drag_mode(self, mode: DragModeStr):
        """Set the behavior for dragging the mouse while the left mouse button is pressed.

        Args:
            mode: drag mode to use

        Raises:
            InvalidParamError: mode does not exist
        """
        if mode not in DRAG_MODE:
            raise InvalidParamError(mode, DRAG_MODE)
        self.setDragMode(DRAG_MODE[mode])

    def get_drag_mode(self) -> DragModeStr:
        """Return current drag mode.

        Returns:
            drag mode
        """
        return DRAG_MODE.inverse[self.dragMode()]

    def set_rubberband_selection_mode(self, mode: ItemSelectionModeStr):
        """Set the behavior for selecting items with a rubber band selection rectangle.

        Args:
            mode: rubberband selection mode to use

        Raises:
            InvalidParamError: mode does not exist
        """
        if mode not in ITEM_SELECTION_MODE:
            raise InvalidParamError(mode, ITEM_SELECTION_MODE)
        self.setRubberBandSelectionMode(ITEM_SELECTION_MODE[mode])

    def get_rubberband_selection_mode(self) -> ItemSelectionModeStr:
        """Return current rubberband selection mode.

        Returns:
            rubberband selection mode
        """
        return ITEM_SELECTION_MODE.inverse[self.rubberBandSelectionMode()]

    def set_cache_mode(self, mode: CacheModeStr):
        """Set the cache mode.

        Args:
            mode: cache mode to use

        Raises:
            InvalidParamError: mode does not exist
        """
        if mode not in CACHE_MODES:
            raise InvalidParamError(mode, CACHE_MODES)
        self.setCacheMode(CACHE_MODES[mode])

    def get_cache_mode(self) -> CacheModeStr:
        """Return current cache mode.

        Returns:
            cache mode
        """
        return CACHE_MODES.inverse[self.cacheMode()]

    def set_optimization_flags(self, *items: str):
        for item in items:
            if item not in OPTIMIZATION_FLAGS:
                raise InvalidParamError(item, OPTIMIZATION_FLAGS)
        flags = helpers.merge_flags(items, OPTIMIZATION_FLAGS)
        self.setOptimizationFlags(flags)

    def get_optimization_flags(self) -> List[str]:
        return [k for k, v in OPTIMIZATION_FLAGS.items() if v & self.optimizationFlags()]


if __name__ == "__main__":
    app = widgets.app()
    view = GraphicsView()
    scene = widgets.GraphicsScene()
    scene.add_line(core.Line(0, 0, 200, 100))
    view.setScene(scene)
    view.show()
    app.main_loop()



from typing import List, Literal
import contextlib

from qtpy import QtGui, QtCore

from prettyqt import core, gui
from prettyqt.utils import bidict, InvalidParamError, colors

CURSOR_SHAPE = bidict(
    arrow=QtCore.Qt.ArrowCursor,
    uparrow=QtCore.Qt.UpArrowCursor,
    cross=QtCore.Qt.CrossCursor,
    wait=QtCore.Qt.WaitCursor,
    caret=QtCore.Qt.IBeamCursor,
    size_vertical=QtCore.Qt.SizeVerCursor,
    size_horizonal=QtCore.Qt.SizeHorCursor,
    size_topright=QtCore.Qt.SizeBDiagCursor,
    size_topleft=QtCore.Qt.SizeFDiagCursor,
    size_all=QtCore.Qt.SizeAllCursor,
    blank=QtCore.Qt.BlankCursor,
    split_vertical=QtCore.Qt.SplitVCursor,
    split_horizontal=QtCore.Qt.SplitHCursor,
    pointing_hand=QtCore.Qt.PointingHandCursor,
    forbidden=QtCore.Qt.ForbiddenCursor,
    open_hand=QtCore.Qt.OpenHandCursor,
    closed_hand=QtCore.Qt.ClosedHandCursor,
    whats_this=QtCore.Qt.WhatsThisCursor,
    busy=QtCore.Qt.BusyCursor,
    drag_move=QtCore.Qt.DragMoveCursor,
    drag_copy=QtCore.Qt.DragCopyCursor,
    drag_link=QtCore.Qt.DragLinkCursor,
    bitmap=QtCore.Qt.BitmapCursor,
)

CursorShapeStr = Literal[
    "arrow",
    "uparrow",
    "cross",
    "wait",
    "caret",
    "size_vertical",
    "size_horizonal",
    "size_topright",
    "size_topleft",
    "size_all",
    "blank",
    "split_vertical",
    "split_horizontal",
    "pointing_hand",
    "forbidden",
    "open_hand",
    "closed_hand",
    "whats_this",
    "busy",
    "drag_move",
    "drag_copy",
    "drag_link",
    "bitmap",
]

LAYOUT_DIRECTION = bidict(
    left_to_right=QtCore.Qt.LeftToRight,
    right_to_left=QtCore.Qt.RightToLeft,
    auto=QtCore.Qt.LayoutDirectionAuto,
)

LayoutDirectionStr = Literal["left_to_right", "right_to_left", "auto"]

APPLICATION_STATES = bidict(
    suspended=QtCore.Qt.ApplicationSuspended,
    hidden=QtCore.Qt.ApplicationHidden,
    inactive=QtCore.Qt.ApplicationInactive,
    active=QtCore.Qt.ApplicationActive,
)

ApplicationStateStr = Literal["suspended", "hidden", "inactive", "active"]


HIGH_DPI_SCALE_FACTOR_ROUNDING_POLICY = bidict(
    round=QtCore.Qt.HighDpiScaleFactorRoundingPolicy.Round,
    ceil=QtCore.Qt.HighDpiScaleFactorRoundingPolicy.Ceil,
    floor=QtCore.Qt.HighDpiScaleFactorRoundingPolicy.Floor,
    round_prefer_floor=QtCore.Qt.HighDpiScaleFactorRoundingPolicy.RoundPreferFloor,
    pass_through=QtCore.Qt.HighDpiScaleFactorRoundingPolicy.PassThrough,
)

HighDpiScaleFactorRoundingPolicyStr = Literal[
    "round", "ceil", "floor", "round_prefer_floor", "pass_through"
]

QtGui.QGuiApplication.__bases__ = (core.CoreApplication,)


class GuiApplication(QtGui.QGuiApplication):
    @classmethod
    @contextlib.contextmanager
    def override_cursor(cls, cursor: CursorShapeStr):
        cls.set_override_cursor(cursor)
        yield cursor
        cls.restore_override_cursor()

    @classmethod
    def set_override_cursor(cls, cursor: CursorShapeStr):
        cursor = gui.Cursor(CURSOR_SHAPE[cursor])
        cls.setOverrideCursor(cursor)

    @classmethod
    def restore_override_cursor(cls):
        cls.restoreOverrideCursor()

    @classmethod
    def get_clipboard(cls) -> gui.Clipboard:
        return gui.Clipboard(cls.clipboard())

    def set_layout_direction(self, direction: LayoutDirectionStr):
        """Set layout direction.

        Args:
            direction: layout direction

        Raises:
            InvalidParamError: layout direction does not exist
        """
        if direction not in LAYOUT_DIRECTION:
            raise InvalidParamError(direction, LAYOUT_DIRECTION)
        self.setLayoutDirection(LAYOUT_DIRECTION[direction])

    def get_layout_direction(self) -> LayoutDirectionStr:
        """Get the current layout direction.

        Returns:
            layout direction
        """
        return LAYOUT_DIRECTION.inverse[self.layoutDirection()]

    @classmethod
    def set_high_dpi_scale_factor_rounding_policy(
        cls, policy: HighDpiScaleFactorRoundingPolicyStr
    ):
        """Set high dpi scale factor rounding policy.

        Args:
            direction: rounding policy

        Raises:
            InvalidParamError: rounding policy does not exist
        """
        if policy not in HIGH_DPI_SCALE_FACTOR_ROUNDING_POLICY:
            raise InvalidParamError(policy, HIGH_DPI_SCALE_FACTOR_ROUNDING_POLICY)
        cls.setHighDpiScaleFactorRoundingPolicy(
            HIGH_DPI_SCALE_FACTOR_ROUNDING_POLICY[policy]
        )

    @classmethod
    def get_high_dpi_scale_factor_rounding_policy(
        cls,
    ) -> HighDpiScaleFactorRoundingPolicyStr:
        """Get the current high dpi scale factor rounding policy.

        Returns:
            rounding policy
        """
        return HIGH_DPI_SCALE_FACTOR_ROUNDING_POLICY.inverse[
            cls.highDpiScaleFactorRoundingPolicy()
        ]

    @classmethod
    def get_application_state(cls) -> List[ApplicationStateStr]:
        """Get the current application state.

        Returns:
            application state
        """
        return [k for k, v in APPLICATION_STATES.items() if v & cls.applicationState()]

    def get_primary_screen(self) -> gui.Screen:
        return gui.Screen(self.primaryScreen())

    def get_screen_at(self, point: QtCore.QPoint) -> gui.Screen:
        return gui.Screen(self.screenAt(point))

    def get_screens(self) -> List[gui.Screen]:
        return [gui.Screen(i) for i in self.screens()]

    @classmethod
    def get_input_method(cls) -> gui.InputMethod:
        return gui.InputMethod(cls.inputMethod())

    @classmethod
    def copy_to_clipboard(cls, text: str):
        """Sets clipboard to supplied text."""
        cb = cls.clipboard()
        cb.clear(mode=cb.Clipboard)
        cb.setText(text, mode=cb.Clipboard)

    @classmethod
    def get_font(cls) -> gui.Font:
        return gui.Font(cls.font())

    def set_icon(self, icon: gui.icon.IconType):
        """Set the default window icon.

        Args:
            icon: icon to use
        """
        icon = gui.icon.get_icon(icon, color=colors.WINDOW_ICON_COLOR)
        self.setWindowIcon(icon)

    def get_icon(self) -> gui.Icon:
        return gui.Icon(self.windowIcon())

    @classmethod
    def get_palette(cls) -> gui.Palette:
        return gui.Palette(cls.palette())



"""Constants module."""
from typing import Literal

from qtpy import QtCore
from bidict import bidict

DISPLAY_ROLE = QtCore.Qt.DisplayRole
USER_ROLE = QtCore.Qt.UserRole
SORT_ROLE = QtCore.Qt.UserRole + 1
NAME_ROLE = QtCore.Qt.UserRole + 2
EDIT_ROLE = QtCore.Qt.EditRole
BACKGROUND_ROLE = QtCore.Qt.BackgroundRole
FOREGROUND_ROLE = QtCore.Qt.ForegroundRole
TOOLTIP_ROLE = QtCore.Qt.ToolTipRole
STATUSTIP_ROLE = QtCore.Qt.StatusTipRole
DECORATION_ROLE = QtCore.Qt.DecorationRole
CHECKSTATE_ROLE = QtCore.Qt.CheckStateRole
ALIGNMENT_ROLE = QtCore.Qt.TextAlignmentRole
FONT_ROLE = QtCore.Qt.FontRole

ALIGN_LEFT = QtCore.Qt.AlignLeft
ALIGN_RIGHT = QtCore.Qt.AlignRight
ALIGN_H_CENTER = QtCore.Qt.AlignHCenter
ALIGN_JUSTIFY = QtCore.Qt.AlignJustify

ALIGN_TOP = QtCore.Qt.AlignTop
ALIGN_BOTTOM = QtCore.Qt.AlignBottom
ALIGN_V_CENTER = QtCore.Qt.AlignVCenter
ALIGN_BASELINE = QtCore.Qt.AlignBaseline

ALIGN_CENTER = QtCore.Qt.AlignCenter
ALIGN_CENTER_LEFT = QtCore.Qt.AlignVCenter | QtCore.Qt.AlignLeft
ALIGN_CENTER_RIGHT = QtCore.Qt.AlignVCenter | QtCore.Qt.AlignRight
ALIGN_TOP_LEFT = QtCore.Qt.AlignTop | QtCore.Qt.AlignLeft
ALIGN_TOP_RIGHT = QtCore.Qt.AlignTop | QtCore.Qt.AlignRight
ALIGN_TOP_CENTER = QtCore.Qt.AlignTop | QtCore.Qt.AlignHCenter
ALIGN_BOTTOM_LEFT = QtCore.Qt.AlignBottom | QtCore.Qt.AlignLeft
ALIGN_BOTTOM_RIGHT = QtCore.Qt.AlignBottom | QtCore.Qt.AlignRight
ALIGN_BOTTOM_CENTER = QtCore.Qt.AlignBottom | QtCore.Qt.AlignHCenter

HORIZONTAL = QtCore.Qt.Horizontal
VERTICAL = QtCore.Qt.Vertical

ASCENDING = QtCore.Qt.AscendingOrder
DESCENDING = QtCore.Qt.DescendingOrder

DROP_ENABLED = QtCore.Qt.ItemIsDropEnabled
DRAG_ENABLED = QtCore.Qt.ItemIsDragEnabled
IS_ENABLED = QtCore.Qt.ItemIsEnabled
IS_SELECTABLE = QtCore.Qt.ItemIsSelectable
IS_EDITABLE = QtCore.Qt.ItemIsEditable
IS_CHECKABLE = QtCore.Qt.ItemIsUserCheckable
IS_AUTO_TRISTATE = QtCore.Qt.ItemIsAutoTristate
IS_USER_TRISTATE = QtCore.Qt.ItemIsUserTristate
NO_FLAGS = QtCore.Qt.NoItemFlags
NO_CHILDREN = QtCore.Qt.ItemNeverHasChildren

MOVE_ACTION = QtCore.Qt.MoveAction
COPY_ACTION = QtCore.Qt.CopyAction

TEXT_WORD_WRAP = QtCore.Qt.TextWordWrap

CTRL_MOD = QtCore.Qt.ControlModifier

KEY_F11 = QtCore.Qt.Key_F11
KEY_DELETE = QtCore.Qt.Key_Delete


try:
    ALIGNMENTS = bidict(
        left=ALIGN_LEFT,
        right=ALIGN_RIGHT,
        top=ALIGN_TOP,
        bottom=ALIGN_BOTTOM,
        top_left=ALIGN_TOP_LEFT,
        top_right=ALIGN_TOP_RIGHT,
        bottom_left=ALIGN_BOTTOM_LEFT,
        bottom_right=ALIGN_BOTTOM_RIGHT,
        center=ALIGN_CENTER,
    )

    SIDES = bidict(
        left=QtCore.Qt.AlignLeft,
        right=QtCore.Qt.AlignRight,
        top=QtCore.Qt.AlignTop,
        bottom=QtCore.Qt.AlignBottom,
    )
    EDGES = bidict(
        top=QtCore.Qt.TopEdge,
        left=QtCore.Qt.LeftEdge,
        right=QtCore.Qt.RightEdge,
        bottom=QtCore.Qt.BottomEdge,
        top_left=QtCore.Qt.TopEdge | QtCore.Qt.LeftEdge,
        top_right=QtCore.Qt.TopEdge | QtCore.Qt.RightEdge,
        bottom_left=QtCore.Qt.BottomEdge | QtCore.Qt.LeftEdge,
        bottom_right=QtCore.Qt.BottomEdge | QtCore.Qt.RightEdge,
    )

    H_ALIGNMENT = bidict(
        left=QtCore.Qt.AlignLeft,
        right=QtCore.Qt.AlignRight,
        center=QtCore.Qt.AlignHCenter,
        justify=QtCore.Qt.AlignJustify,
    )

    V_ALIGNMENT = bidict(
        top=QtCore.Qt.AlignTop,
        bottom=QtCore.Qt.AlignBottom,
        center=QtCore.Qt.AlignVCenter,
        baseline=QtCore.Qt.AlignBaseline,
    )

except TypeError:
    ALIGNMENTS = SIDES = EDGES = H_ALIGNMENT = V_ALIGNMENT = bidict()

EdgeStr = Literal[
    "top",
    "left",
    "right",
    "bottom",
    "top_left",
    "top_right",
    "bottom_left",
    "bottom_right",
]

AlignmentStr = Literal[
    "top",
    "left",
    "right",
    "bottom",
    "top_left",
    "top_right",
    "bottom_left",
    "bottom_right",
]

SideStr = Literal["left", "right", "top", "bottom"]

HorizontalAlignmentStr = Literal[
    "left",
    "right",
    "center",
    "justify",
]
VerticalAlignmentStr = Literal[
    "top",
    "bottom",
    "center",
    "baseline",
]

ORIENTATION = bidict(horizontal=QtCore.Qt.Horizontal, vertical=QtCore.Qt.Vertical)

OrientationStr = Literal["horizontal", "vertical"]

STATE = bidict(
    unchecked=QtCore.Qt.Unchecked,
    partial=QtCore.Qt.PartiallyChecked,
    checked=QtCore.Qt.Checked,
)

StateStr = Literal["unchecked", "partial", "checked"]

MATCH_FLAGS = dict(
    exact=QtCore.Qt.MatchExactly,
    contains=QtCore.Qt.MatchContains,
    starts_with=QtCore.Qt.MatchStartsWith,
    ends_with=QtCore.Qt.MatchEndsWith,
    wildcard=QtCore.Qt.MatchWildcard,
    regex=QtCore.Qt.MatchRegExp,
)

MatchFlagStr = Literal[
    "exact", "containts", "starts_with", "ends_with", "wildcard", "regex"
]

FILTER_MODE = bidict(
    starts_with=QtCore.Qt.MatchStartsWith,
    contains=QtCore.Qt.MatchContains,
    ends_with=QtCore.Qt.MatchEndsWith,
)

FilterModeStr = Literal["starts_with", "contains", "ends_with"]



from typing import Optional, List, Callable, Union
from dataclasses import dataclass
import logging

from qtpy import QtGui, QtCore

from prettyqt import core, constants


logger = logging.getLogger(__name__)

SMALL_COL_WIDTH = 120
MEDIUM_COL_WIDTH = 200


@dataclass(frozen=True)
class ColumnItem:
    """Determines how an object attribute is shown."""

    name: str
    label: Optional[Callable]
    checkstate: Optional[Callable] = None
    doc: str = "<no help available>"
    col_visible: bool = True
    width: Union[int, str] = SMALL_COL_WIDTH
    alignment: Optional[Union[Callable, int]] = None
    line_wrap: str = "none"
    foreground_color: Optional[Union[Callable, str]] = None
    background_color: Optional[Union[Callable, str]] = None
    decoration: Optional[Union[Callable, QtGui.QIcon]] = None
    font: Optional[Union[Callable, QtGui.QFont]] = None
    selectable: bool = True
    enabled: bool = True
    editable: bool = False
    checkable: bool = False
    tristate: bool = False

    def get_name(self) -> str:
        return self.name

    def get_flag(self):
        flag = constants.NO_FLAGS
        if self.selectable:
            flag |= constants.IS_SELECTABLE
        if self.enabled:
            flag |= constants.IS_ENABLED
        if self.editable:
            flag |= constants.IS_EDITABLE
        if self.checkable:
            flag |= constants.IS_CHECKABLE
        if self.tristate:
            flag |= constants.IS_USER_TRISTATE
        return flag

    def get_label(self, tree_item):
        if self.label is None:
            return ""
        elif callable(self.label):
            return self.label(tree_item)
        return self.label

    def get_checkstate(self, tree_item):
        if self.checkstate is None:
            return None
        elif callable(self.checkstate):
            return self.checkstate(tree_item)
        return self.checkstate

    def get_font(self, tree_item):
        if self.font is None:
            return None
        elif callable(self.font):
            return self.font(tree_item)
        return self.font

    def get_foreground_color(self, tree_item):
        if self.foreground_color is None:
            return None
        elif callable(self.foreground_color):
            return self.foreground_color(tree_item)
        return self.foreground_color

    def get_background_color(self, tree_item):
        if self.background_color is None:
            return None
        elif callable(self.background_color):
            return self.background_color(tree_item)
        return self.background_color

    def get_decoration(self, tree_item):
        if self.decoration is None:
            return None
        elif callable(self.decoration):
            return self.decoration(tree_item)
        return self.decoration

    def get_alignment(self, tree_item) -> int:
        if self.alignment is None:
            return constants.ALIGN_LEFT
        elif callable(self.alignment):
            return self.alignment(tree_item)
        elif isinstance(self.alignment, str):
            return constants.ALIGNMENTS[self.alignment]
        return self.alignment

    def get_width(self) -> int:
        if self.width == "small":
            return SMALL_COL_WIDTH
        elif self.width == "medium":
            return MEDIUM_COL_WIDTH
        elif isinstance(self.width, int):
            return self.width
        raise ValueError(self.width)


class ColumnItemModel(core.AbstractItemModel):
    """Model that provides an interface to an objectree that is build of TreeItems."""

    def __init__(
        self,
        attr_cols: Optional[List[ColumnItem]] = None,
        parent: Optional[QtCore.QObject] = None,
    ):
        super().__init__(parent)
        self._attr_cols = attr_cols if attr_cols is not None else []

    def columnCount(self, _parent=None):
        """Return the number of columns in the tree."""
        return len(self._attr_cols)

    def data(self, index, role):
        """Return the tree item at the given index and role."""
        if not index.isValid():
            return None

        col = index.column()
        tree_item = index.internalPointer()

        if role in [constants.DISPLAY_ROLE, constants.EDIT_ROLE]:
            func = self._attr_cols[col].label
            if func is None:
                return ""
            attr = func(tree_item)
            return attr.replace("\n", " ")
        elif role == constants.DECORATION_ROLE:
            return self._attr_cols[col].get_decoration(tree_item)
        elif role == constants.CHECKSTATE_ROLE:
            return self._attr_cols[col].get_checkstate(tree_item)
        elif role == constants.ALIGNMENT_ROLE:
            return self._attr_cols[col].get_alignment(tree_item)
        elif role == constants.FOREGROUND_ROLE:
            return self._attr_cols[col].get_foreground_color(tree_item)
        elif role == constants.BACKGROUND_ROLE:
            return self._attr_cols[col].get_background_color(tree_item)
        elif role == constants.FONT_ROLE:
            return self._attr_cols[col].get_font(tree_item)
        else:
            return None

    def flags(self, index):
        if not index.isValid():
            return constants.NO_CHILDREN
        col = index.column()
        return self._attr_cols[col].get_flag()

    def headerData(self, section, orientation, role):
        if orientation == constants.HORIZONTAL and role == constants.DISPLAY_ROLE:
            return self._attr_cols[section].name
        else:
            return None



import logging

from qtpy import PYQT5, PYSIDE2

if PYQT5:
    from PyQt5 import QtTextToSpeech  # type: ignore
elif PYSIDE2:
    from PySide2 import QtTextToSpeech


from prettyqt.utils import bidict

logger = logging.getLogger()

AGE = bidict(
    child=QtTextToSpeech.QVoice.Child,
    teenager=QtTextToSpeech.QVoice.Teenager,
    adult=QtTextToSpeech.QVoice.Adult,
    senior=QtTextToSpeech.QVoice.Senior,
    other=QtTextToSpeech.QVoice.Other,
)

GENDER = bidict(
    male=QtTextToSpeech.QVoice.Male,
    female=QtTextToSpeech.QVoice.Female,
    unknown=QtTextToSpeech.QVoice.Unknown,
)


class Voice(QtTextToSpeech.QVoice):
    def get_age(self) -> str:
        return AGE.inverse[self.age()]

    def get_gender(self) -> str:
        return GENDER.inverse[self.gender()]


if __name__ == "__main__":
    val = Voice()



from typing import Optional

from qtpy import QtMultimedia


class CameraInfo(QtMultimedia.QCameraInfo):
    def __repr__(self):
        return f"{type(self).__name__}({self.deviceName()!r})"

    @classmethod
    def get_cameras(cls):
        return iter(cls(i) for i in cls.availableCameras())

    @classmethod
    def get_camera(cls, name: Optional[str] = None):
        if name is None:
            return cls(cls.defaultCamera())
        for cam in cls.get_cameras():
            if cam.deviceName() == name:
                return cam


if __name__ == "__main__":
    caminfo = CameraInfo()
    cams = caminfo.get_camera()
    print(cams)



from qtpy import QtMultimedia

from prettyqt import core, multimedia, gui
from prettyqt.utils import bidict, InvalidParamError

FIELD_TYPES = bidict(
    progressive_frame=QtMultimedia.QVideoFrame.ProgressiveFrame,
    top_field=QtMultimedia.QVideoFrame.TopField,
    bottom_field=QtMultimedia.QVideoFrame.BottomField,
    interlaced_frame=QtMultimedia.QVideoFrame.InterlacedFrame,
)

PIXEL_FORMAT = bidict(
    invalid=QtMultimedia.QVideoFrame.Format_Invalid,
    argb32=QtMultimedia.QVideoFrame.Format_ARGB32,
    argb32_premultiplied=QtMultimedia.QVideoFrame.Format_ARGB32_Premultiplied,
    rgb32=QtMultimedia.QVideoFrame.Format_RGB32,
    rgb24=QtMultimedia.QVideoFrame.Format_RGB24,
    rgb565=QtMultimedia.QVideoFrame.Format_RGB565,
    rgb555=QtMultimedia.QVideoFrame.Format_RGB555,
    argb8565_premultiplied=QtMultimedia.QVideoFrame.Format_ARGB8565_Premultiplied,
    bgra32=QtMultimedia.QVideoFrame.Format_BGRA32,
    bgra32_premultiplied=QtMultimedia.QVideoFrame.Format_BGRA32_Premultiplied,
    abgr32=QtMultimedia.QVideoFrame.Format_ABGR32,
    bgr32=QtMultimedia.QVideoFrame.Format_BGR32,
    bgr24=QtMultimedia.QVideoFrame.Format_BGR24,
    bgr565=QtMultimedia.QVideoFrame.Format_BGR565,
    bgr555=QtMultimedia.QVideoFrame.Format_BGR555,
    bgra5658_premultiplied=QtMultimedia.QVideoFrame.Format_BGRA5658_Premultiplied,
    ayuv444=QtMultimedia.QVideoFrame.Format_AYUV444,
    yuv444_premultiplied=QtMultimedia.QVideoFrame.Format_AYUV444_Premultiplied,
    yuv444=QtMultimedia.QVideoFrame.Format_YUV444,
    yuv420p=QtMultimedia.QVideoFrame.Format_YUV420P,
    yuv422p=QtMultimedia.QVideoFrame.Format_YUV422P,
    yv12=QtMultimedia.QVideoFrame.Format_YV12,
    uyvy=QtMultimedia.QVideoFrame.Format_UYVY,
    yuyv=QtMultimedia.QVideoFrame.Format_YUYV,
    nv12=QtMultimedia.QVideoFrame.Format_NV12,
    nv21=QtMultimedia.QVideoFrame.Format_NV21,
    imc1=QtMultimedia.QVideoFrame.Format_IMC1,
    imc2=QtMultimedia.QVideoFrame.Format_IMC2,
    imc3=QtMultimedia.QVideoFrame.Format_IMC3,
    imc4=QtMultimedia.QVideoFrame.Format_IMC4,
    y8=QtMultimedia.QVideoFrame.Format_Y8,
    y16=QtMultimedia.QVideoFrame.Format_Y16,
    jpeg=QtMultimedia.QVideoFrame.Format_Jpeg,
    camera_raw=QtMultimedia.QVideoFrame.Format_CameraRaw,
    adobe_dng=QtMultimedia.QVideoFrame.Format_AdobeDng,
    user=QtMultimedia.QVideoFrame.Format_User,
)

HANDLE_TYPES = multimedia.abstractvideobuffer.HANDLE_TYPES
MAP_MODE = multimedia.abstractvideobuffer.MAP_MODE


class VideoFrame(QtMultimedia.QVideoFrame):
    def __repr__(self):
        return (
            f"{type(self).__name__}({self.mappedBytes()}, {self.get_size()},"
            f"{self.bytesPerLine()}, {self.pixelFormat()})"
        )

    def get_handle_type(self) -> str:
        return HANDLE_TYPES.inverse[self.handleType()]

    def get_map_mode(self) -> str:
        return MAP_MODE.inverse[self.mapMode()]

    def get_size(self) -> core.Size:
        return core.Size(self.size())

    def get_image(self) -> gui.Image:
        return gui.Image(self.image())

    def get_pixel_format(self) -> str:
        return PIXEL_FORMAT.inverse[self.pixelFormat()]

    def get_field_type(self) -> str:
        """Set field type.

        Valid values: "progressive_frame", "top_field", "bottom_field",
                      "interlaced_frame"

        """
        return FIELD_TYPES.inverse[self.fieldType()]

    def set_field_type(self, typ: str):
        """Return current field type.

        Possible values: "progressive_frame", "top_field", "bottom_field",
                         "interlaced_frame"
        Args:
            typ: capture mode

        Raises:
            InvalidParamError: capture mode does not exist
        """
        if typ not in FIELD_TYPES:
            raise InvalidParamError(typ, FIELD_TYPES)
        self.setFieldType(FIELD_TYPES[typ])


if __name__ == "__main__":
    frame = VideoFrame()



"""Module containing custom widget classes."""

from .image import Image
from .listinput import ListInput
from .booldicttoolbutton import BoolDictToolButton
from .optionalwidget import OptionalWidget
from .collapsibleframe import CollapsibleFrame
from .expandableline import ExpandableLine
from .singlelinetextedit import SingleLineTextEdit
from .regexinput import RegexInput
from .mappedcheckbox import MappedCheckBox
from .promptlineedit import PromptLineEdit
from .logtextedit import LogTextEdit
from .flagselectionwidget import FlagSelectionWidget
from .stringornumberwidget import StringOrNumberWidget
from .iconlabel import IconLabel
from .flowlayout import FlowLayout
from .sidebarwidget import SidebarWidget
from .colorchooserbutton import ColorChooserButton
from .filechooserbutton import FileChooserButton
from .fontchooserbutton import FontChooserButton
from .inputandslider import InputAndSlider
from .spanslider import SpanSlider
from .labeledslider import LabeledSlider
from .waitingspinner import WaitingSpinner
from .markdownwidget import MarkdownWindow
from .imageviewer import ImageViewer
from .popupinfo import PopupInfo
from .selectionwidget import SelectionWidget
from .codeeditor import CodeEditor
from .roundprogressbar import RoundProgressBar

# from .stareditor import StarEditor, StarRating
from .timeline import Timeline, VideoSample
from .player import Player
from .regexeditor.editor import RegexEditorWidget

# Deprecated: should be imported from custom_delegates instead
from prettyqt.custom_delegates.buttondelegate import ButtonDelegate
from prettyqt.custom_delegates.radiodelegate import RadioDelegate

__all__ = [
    "Image",
    "ListInput",
    "BoolDictToolButton",
    "OptionalWidget",
    "CollapsibleFrame",
    "ExpandableLine",
    "SingleLineTextEdit",
    "RegexInput",
    "MappedCheckBox",
    "PromptLineEdit",
    "LogTextEdit",
    "FlagSelectionWidget",
    "StringOrNumberWidget",
    "IconLabel",
    "FlowLayout",
    "SidebarWidget",
    "ColorChooserButton",
    "FileChooserButton",
    "FontChooserButton",
    "InputAndSlider",
    "SpanSlider",
    "LabeledSlider",
    "WaitingSpinner",
    "RoundProgressBar",
    "PopupInfo",
    "ButtonDelegate",
    "RadioDelegate",
    "SelectionWidget",
    "ImageViewer",
    "MarkdownWindow",
    "CodeEditor",
    "Player",
    "Timeline",
    # "StarEditor",
    # "StarRating",
    "VideoSample",
    "RegexEditorWidget",
]



from typing import Union

from qtpy import QtWidgets

from prettyqt import widgets


QtWidgets.QSpacerItem.__bases__ = (widgets.LayoutItem,)


class SpacerItem(QtWidgets.QSpacerItem):
    def __init__(
        self,
        w: int,
        h: int,
        h_policy: Union[int, widgets.sizepolicy.SizePolicyStr],
        v_policy: Union[int, widgets.sizepolicy.SizePolicyStr],
    ):
        if h_policy in widgets.sizepolicy.SIZE_POLICY:
            h_policy = widgets.sizepolicy.SIZE_POLICY[h_policy]
        if v_policy in widgets.sizepolicy.SIZE_POLICY:
            v_policy = widgets.sizepolicy.SIZE_POLICY[v_policy]
        super().__init__(w, h, h_policy, v_policy)

    def change_size(
        self,
        w: int,
        h: int,
        h_policy: widgets.sizepolicy.SizePolicyStr = "minimum",
        v_policy: widgets.sizepolicy.SizePolicyStr = "minimum",
    ):
        if h_policy in widgets.sizepolicy.SIZE_POLICY:
            h_policy = widgets.sizepolicy.SIZE_POLICY[h_policy]
        if v_policy in widgets.sizepolicy.SIZE_POLICY:
            v_policy = widgets.sizepolicy.SIZE_POLICY[v_policy]
        self.changeSize(w, h, h_policy, v_policy)



import sys
import traceback
from typing import Optional, List

from qtpy import QtCore, QtWidgets

from prettyqt import gui, widgets, core
from prettyqt.utils import bidict, InvalidParamError


ICONS = bidict(
    none=QtWidgets.QMessageBox.NoIcon,
    information=QtWidgets.QMessageBox.Information,
    warning=QtWidgets.QMessageBox.Warning,
    critical=QtWidgets.QMessageBox.Critical,
    question=QtWidgets.QMessageBox.Question,
)

BUTTONS = bidict(
    none=QtWidgets.QMessageBox.NoButton,
    cancel=QtWidgets.QMessageBox.Cancel,
    ok=QtWidgets.QMessageBox.Ok,
    save=QtWidgets.QMessageBox.Save,
    open=QtWidgets.QMessageBox.Open,
    close=QtWidgets.QMessageBox.Close,
    discard=QtWidgets.QMessageBox.Discard,
    apply=QtWidgets.QMessageBox.Apply,
    reset=QtWidgets.QMessageBox.Reset,
    restore_defaults=QtWidgets.QMessageBox.RestoreDefaults,
    help=QtWidgets.QMessageBox.Help,
    save_all=QtWidgets.QMessageBox.SaveAll,
    yes=QtWidgets.QMessageBox.Yes,
    yes_to_all=QtWidgets.QMessageBox.YesToAll,
    no=QtWidgets.QMessageBox.No,
    no_to_all=QtWidgets.QMessageBox.NoToAll,
    abort=QtWidgets.QMessageBox.Abort,
    retry=QtWidgets.QMessageBox.Retry,
    ignore=QtWidgets.QMessageBox.Ignore,
)

TEXT_FORMAT = bidict(
    rich=QtCore.Qt.RichText, plain=QtCore.Qt.PlainText, auto=QtCore.Qt.AutoText
)
if core.VersionNumber.get_qt_version() >= (5, 14, 0):
    TEXT_FORMAT["markdown"] = QtCore.Qt.MarkdownText

QtWidgets.QMessageBox.__bases__ = (widgets.BaseDialog,)


class MessageBox(QtWidgets.QMessageBox):
    def __init__(
        self,
        icon: gui.icon.IconType = None,
        title: Optional[str] = None,
        text: str = "",
        informative_text: str = "",
        details: str = "",
        buttons: Optional[list] = None,
        parent: Optional[QtWidgets.QWidget] = None,
    ):
        super().__init__(parent)
        self.set_icon(icon)
        self.setText(text)
        self.setInformativeText(informative_text)
        self.setWindowTitle(title)
        self.setWindowFlags(
            QtCore.Qt.Dialog | QtCore.Qt.WindowTitleHint | QtCore.Qt.CustomizeWindowHint
        )
        self.setDetailedText(details)
        if isinstance(buttons, list):
            for b in buttons:
                self.add_button(b)

    @classmethod
    def message(
        cls,
        text: str,
        title: str = None,
        icon: gui.icon.IconType = None,
        detail_text: Optional[str] = None,
    ) -> str:
        m = cls("none", title, text)
        m.set_icon(icon)
        if detail_text is not None:
            m.setDetailedText(detail_text)
        return m.show_blocking()

    @classmethod
    def show_exception(cls, exception: Exception):
        exctype, value = sys.exc_info()[:2]
        tb = traceback.format_exc()
        dlg = cls(text=str(value), title=str(exctype), icon="critical", details=tb)
        dlg.show_blocking()

    def set_icon(self, icon: gui.icon.IconType):
        if icon in ICONS:
            self.setIcon(ICONS[icon])
            return None
        icon = gui.icon.get_icon(icon)
        self.setIconPixmap(icon.get_pixmap(size=64))

    def show_blocking(self) -> str:
        return BUTTONS.inverse[self.exec_()]

    def get_standard_buttons(self) -> List[str]:
        return [k for k, v in BUTTONS.items() if v & self.standardButtons()]

    def add_button(self, button: str) -> QtWidgets.QPushButton:
        """Add a default button.

        Valid arguments: "none", "cancel", "ok", "save", "open", "close",
                         "discard", "apply", "reset", "restore_defaults",
                         "help", "save_all", "yes", "yes_to_all", "no",
                         "no_to_all", "abort", "retry", "ignore"

        Args:
            button: button to add

        Returns:
            created button

        Raises:
            InvalidParamError: Button type not available
        """
        if button not in BUTTONS:
            raise InvalidParamError(button, BUTTONS)
        return self.addButton(BUTTONS[button])

    # @classmethod
    # def show_exception(cls, exception):
    #     header = str(exception[0])
    #     error_text = str(exception[1])
    #     widgets.MessageBox.message(error_text, header, "mdi.exclamation")

    def set_text_format(self, text_format: str):
        """Set the text format.

        Allowed values are "rich", "plain", "auto"

        Args:
            text_format: text format to use

        Raises:
            InvalidParamError: text format does not exist
        """
        if text_format not in TEXT_FORMAT:
            raise InvalidParamError(text_format, TEXT_FORMAT)
        self.setTextFormat(TEXT_FORMAT[text_format])

    def get_text_format(self) -> str:
        """Return current text format.

        Possible values: "rich", "plain", "auto"

        Returns:
            text format
        """
        return TEXT_FORMAT.inverse[self.textFormat()]


if __name__ == "__main__":
    app = widgets.app()
    ret = MessageBox(icon="warning", title="header", text="text", details="details")
    ret.set_icon("mdi.folder")
    ret.show()
    print(ret)
    app.main_loop()



from typing import Literal

from qtpy import QtGui, QtCore

from prettyqt import core, gui
from prettyqt.utils import colors
from prettyqt.utils import bidict, InvalidParamError

PEN_STYLE = bidict(
    none=QtCore.Qt.NoPen,
    solid=QtCore.Qt.SolidLine,
    dash=QtCore.Qt.DashLine,
    dot=QtCore.Qt.DotLine,
    dash_dot=QtCore.Qt.DashDotLine,
    dash_dot_dot=QtCore.Qt.DashDotDotLine,
    custom_dash=QtCore.Qt.CustomDashLine,
)

PenStyleStr = Literal[
    "none", "solid", "dash", "dot", "dash_dot", "dash_dot_dot", "custom_dash"
]

CAP_STYLE = bidict(
    flat=QtCore.Qt.FlatCap, square=QtCore.Qt.SquareCap, round=QtCore.Qt.RoundCap
)

CapStyleStr = Literal["flat", "square", "round"]

JOIN_STYLE = bidict(
    miter=QtCore.Qt.MiterJoin,
    bevel=QtCore.Qt.BevelJoin,
    round=QtCore.Qt.RoundJoin,
    svg_miter=QtCore.Qt.SvgMiterJoin,
)

JoinStyleStr = Literal["miter", "bevel", "round" "svg_miter"]


class Pen(QtGui.QPen):
    def __getstate__(self):
        return bytes(self)

    def __setstate__(self, ba):
        core.DataStream.write_bytearray(ba, self)

    def __bytes__(self):
        ba = core.DataStream.create_bytearray(self)
        return bytes(ba)

    def __reduce__(self):
        return type(self), (), self.__getstate__()

    def __repr__(self):
        return f"{type(self).__name__}({self.get_color()})"

    def set_color(self, color: colors.ColorType):
        color = colors.get_color(color)
        self.setColor(color)

    def get_color(self) -> gui.Color:
        return gui.Color(self.color())

    def get_brush(self) -> gui.Brush:
        return gui.Brush(self.brush())

    def set_cap_style(self, style: CapStyleStr):
        """Set cap style to use.

        Allowed values are "flat", "square", "round"

        Args:
            style: cap style to use

        Raises:
            InvalidParamError: cap style does not exist
        """
        if style not in CAP_STYLE:
            raise InvalidParamError(style, CAP_STYLE)
        self.setCapStyle(CAP_STYLE[style])

    def get_cap_style(self) -> CapStyleStr:
        """Return current cap style.

        Possible values: "flat", "square", "round"

        Returns:
            cap style
        """
        return CAP_STYLE.inverse[self.capStyle()]

    def set_join_style(self, style: JoinStyleStr):
        """Set join style to use.

        Allowed values are "miter", "bevel", "round", "svg_miter"

        Args:
            style: join style to use

        Raises:
            InvalidParamError: join style does not exist
        """
        if style not in JOIN_STYLE:
            raise InvalidParamError(style, JOIN_STYLE)
        self.setJoinStyle(JOIN_STYLE[style])

    def get_join_style(self) -> JoinStyleStr:
        """Return current join style.

        Possible values: "miter", "bevel", "round", "svg_miter"

        Returns:
            join style
        """
        return JOIN_STYLE.inverse[self.joinStyle()]

    def set_style(self, style: str):
        """Set pen style to use.

        Allowed values are "none", "solid", "dash", "dot", "dash_dot", "dash_dot_dot",
        "custom_dash"

        Args:
            style: pen style to use

        Raises:
            InvalidParamError: pen style does not exist
        """
        if style not in PEN_STYLE:
            raise InvalidParamError(style, PEN_STYLE)
        self.setStyle(PEN_STYLE[style])

    def get_style(self) -> str:
        """Return current pen style.

        Possible values: "none", "solid", "dash", "dot", "dash_dot", "dash_dot_dot",
        "custom_dash"

        Returns:
            pen style
        """
        return PEN_STYLE.inverse[self.style()]



from typing import Optional
from qtpy import QtWidgets

from prettyqt import widgets, constants
from prettyqt.utils import bidict, InvalidParamError


TEXT_DIRECTIONS = bidict(
    top_to_bottom=QtWidgets.QProgressBar.TopToBottom,
    bottom_to_top=QtWidgets.QProgressBar.BottomToTop,
)


QtWidgets.QProgressBar.__bases__ = (widgets.Widget,)


class ProgressBar(QtWidgets.QProgressBar):
    """Progress dialog.

    wrapper for QtWidgets.QProgressBar
    """

    def __init__(
        self, text_visible: bool = True, parent: Optional[QtWidgets.QWidget] = None
    ):
        super().__init__(parent=parent)
        self.setTextVisible(text_visible)

    def serialize_fields(self):
        return dict(
            alignment=self.get_alignment(),
            format=self.format(),
            # inverted_appearance=self.invertedAppearance(),
            minimum=self.minimum(),
            maximum=self.maximum(),
            orientation=self.get_orientation(),
            text=self.text(),
            # text_direction=self.get_text_direction(),
            text_visible=self.isTextVisible(),
            value=self.value(),
        )

    def set_alignment(self, alignment: constants.AlignmentStr):
        """Set the alignment of the layout.

        Args:
            alignment: alignment for the layout

        Raises:
            InvalidParamError: alignment does not exist
        """
        if alignment not in constants.ALIGNMENTS:
            raise InvalidParamError(alignment, constants.ALIGNMENTS)
        self.setAlignment(constants.ALIGNMENTS[alignment])

    def get_alignment(self) -> str:
        """Return current alignment.

        Returns:
            alignment
        """
        return constants.ALIGNMENTS.inverse[self.alignment()]

    def set_text_direction(self, text_direction: str):
        """Set the text direction of the layout.

        Allowed values are "top_to_bottom", "bottom_to_top"

        Args:
            text_direction: text direction for the layout

        Raises:
            InvalidParamError: text direction does not exist
        """
        if text_direction not in TEXT_DIRECTIONS:
            raise InvalidParamError(text_direction, TEXT_DIRECTIONS)
        self.setTextDirection(TEXT_DIRECTIONS[text_direction])

    def get_text_direction(self) -> str:
        """Return current text direction.

        Possible values are "top_to_bottom", "bottom_to_top"

        Returns:
            Text direction
        """
        return TEXT_DIRECTIONS.inverse[self.textDirection()]

    def set_orientation(self, orientation: constants.OrientationStr):
        """Set the orientation of the progress bar.

        Args:
            orientation: orientation for the progress bar

        Raises:
            InvalidParamError: orientation does not exist
        """
        if orientation not in constants.ORIENTATION:
            raise InvalidParamError(orientation, constants.ORIENTATION)
        self.setOrientation(constants.ORIENTATION[orientation])

    def get_orientation(self) -> constants.OrientationStr:
        """Return current orientation.

        Returns:
            orientation
        """
        return constants.ORIENTATION.inverse[self.orientation()]

    def set_range(self, start: int, end: int):
        self.setRange(start, end)


if __name__ == "__main__":
    app = widgets.app()
    widget = ProgressBar()
    widget.show()
    app.main_loop()



from qtpy import PYQT5, PYSIDE2

if PYQT5:
    from PyQt5 import QtWebEngineWidgets  # type: ignore
elif PYSIDE2:
    from PySide2 import QtWebEngineWidgets

from prettyqt import core


class WebEngineHistoryItem(QtWebEngineWidgets.QWebEngineHistoryItem):
    def get_url(self) -> core.Url:
        return core.Url(self.url())

    def get_icon_url(self) -> core.Url:
        return core.Url(self.iconUrl())

    def get_last_visited(self) -> core.DateTime:
        return core.DateTime(self.lastVisited())



from qtpy import QtWidgets, QtGui

from prettyqt import widgets, gui
from prettyqt.utils import InvalidParamError


QtWidgets.QWizardPage.__bases__ = (widgets.Widget,)


class WizardPage(QtWidgets.QWizardPage):
    def set_pixmap(self, typ: str, pixmap: QtGui.QPixmap):
        if typ not in widgets.wizard.WIZARD_PIXMAP:
            raise InvalidParamError(typ, widgets.wizard.WIZARD_PIXMAP)
        self.setPixmap(widgets.wizard.WIZARD_PIXMAP[typ], pixmap)

    def get_pixmap(self, typ: str) -> gui.Pixmap:
        if typ not in widgets.wizard.WIZARD_PIXMAP:
            raise InvalidParamError(typ, widgets.wizard.WIZARD_PIXMAP)
        return gui.Pixmap(self.pixmap(widgets.wizard.WIZARD_PIXMAP[typ]))

    def set_button_text(self, button_type: str, value: str) -> str:
        """Set text for given button type.

        Args:
            button_type: button to get text from
            value: text to set

        """
        if button_type not in widgets.wizard.WIZARD_BUTTON:
            raise InvalidParamError(button_type, widgets.wizard.WIZARD_BUTTON)
        return self.setButtonText(widgets.wizard.WIZARD_BUTTON[button_type], value)

    def get_button_text(self, button_type: str) -> str:
        """Return text for given button type.

        Args:
            button_type: button to get text from

        Returns:
            Button text
        """
        if button_type not in widgets.wizard.WIZARD_BUTTON:
            raise InvalidParamError(button_type, widgets.wizard.WIZARD_BUTTON)
        return self.buttonText(widgets.wizard.WIZARD_BUTTON[button_type])



from typing import Union, Optional, Literal

from qtpy import QtCore

from prettyqt import core
from prettyqt.utils import bidict, InvalidParamError


NAME_TYPE = bidict(
    default=QtCore.QTimeZone.DefaultName,
    long=QtCore.QTimeZone.LongName,
    short=QtCore.QTimeZone.ShortName,
    offset=QtCore.QTimeZone.OffsetName,
)

NameTypeStr = Literal["default", "long", "short", "offset"]

TIME_TYPE = bidict(
    standard=QtCore.QTimeZone.StandardTime,
    daylight=QtCore.QTimeZone.DaylightTime,
    generic=QtCore.QTimeZone.GenericTime,
)

TimeTypeStr = Literal["standard", "daylight", "generic"]


class TimeZone(QtCore.QTimeZone):
    def __init__(self, *args, **kwargs):
        if len(args) == 1 and isinstance(args[0], str):
            super().__init__(args[0].encode())
        else:
            super().__init__(*args, **kwargs)

    def __repr__(self):
        return f"{type(self).__name__}({self.get_id()!r})"

    def __str__(self):
        return self.get_id()

    def __reduce__(self):
        return self.__class__, (self.get_id(),)

    def get_id(self) -> str:
        return bytes(self.id()).decode()

    def get_display_name(
        self,
        datetime: Union[QtCore.QDateTime, TimeTypeStr],
        name_type: NameTypeStr = "default",
        locale: Optional[core.Locale] = None,
    ) -> str:
        if isinstance(datetime, str):
            if datetime not in TIME_TYPE:
                raise InvalidParamError(datetime, TIME_TYPE)
            datetime = TIME_TYPE[datetime]
        if name_type not in NAME_TYPE:
            raise InvalidParamError(name_type, NAME_TYPE)
        if locale is None:
            locale = core.Locale()
        return self.displayName(datetime, NAME_TYPE[name_type], locale)

    # def get_value(self) -> datetime.datetime:
    #     try:
    #         return self.toPython()
    #     except TypeError:
    #         return self.toPyTimeZone()


if __name__ == "__main__":
    date = core.TimeZone(2000, 11, 11)
    dt = TimeZone(date)



from typing import List

from qtpy import PYQT5, PYSIDE2

if PYQT5:
    from PyQt5 import QtPositioning
elif PYSIDE2:
    from PySide2 import QtPositioning

from prettyqt import core
from prettyqt.utils import bidict, InvalidParamError, helpers

QGeoPositionInfoSource = QtPositioning.QGeoPositionInfoSource

POSITIONING_METHODS = bidict(
    none=QGeoPositionInfoSource.NoPositioningMethods,
    satellite=QGeoPositionInfoSource.SatellitePositioningMethods,
    non_satellite=QGeoPositionInfoSource.NonSatellitePositioningMethods,
    all=QGeoPositionInfoSource.AllPositioningMethods,
)

ERRORS = bidict(
    access_error=QtPositioning.QGeoPositionInfoSource.AccessError,
    closed_error=QtPositioning.QGeoPositionInfoSource.ClosedError,
    none=QtPositioning.QGeoPositionInfoSource.NoError,
    unknown_source=QtPositioning.QGeoPositionInfoSource.UnknownSourceError,
)

QtPositioning.QGeoPositionInfoSource.__bases__ = (core.Object,)


class GeoPositionInfoSource(QtPositioning.QGeoPositionInfoSource):
    def serialize_fields(self):
        return dict(
            minimum_update_interval=self.minimumUpdateInterval(),
            source_name=self.sourceName(),
            update_interval=self.updateInterval(),
        )

    def __str__(self):
        return self.sourceName()

    def __repr__(self):
        return f"{type(self).__name__}()"

    def get_error(self) -> str:
        """Return error type.

        possible values are "access_error" "closed_error", "none", "unkown_source"

        Returns:
            error type
        """
        return ERRORS.inverse[self.error()]

    def set_preferred_positioning_methods(self, *methods: str):
        """Set preferred positioning methods.

        valid values are "none", "satellite", "non_satellite", "all"

        Args:
            methods: positioning methods to use

        Raises:
            InvalidParamError: invalid positioning methods
        """
        for item in methods:
            if item not in POSITIONING_METHODS:
                raise InvalidParamError(item, POSITIONING_METHODS)
        flags = helpers.merge_flags(methods, POSITIONING_METHODS)
        self.setPreferredPositioningMethods(flags)

    def get_preferred_positioning_methods(self) -> List[str]:
        """Return list of preferred positioning methods.

        possible included values are "none", "satellite", "non_satellite", "all"

        Returns:
            list of preferred positioning methods
        """
        return [
            k
            for k, v in POSITIONING_METHODS.items()
            if v & self.preferredPositioningMethods()
        ]

    def get_supported_positioning_methods(self) -> List[str]:
        """Return list of supported positioning methods.

        possible included values are "none", "satellite", "non_satellite", "all"

        Returns:
            list of supported positioning methods
        """
        return [
            k
            for k, v in POSITIONING_METHODS.items()
            if v & self.supportedPositioningMethods()
        ]



from qtpy import QtGui

from prettyqt import gui

QtGui.QTextImageFormat.__bases__ = (gui.TextCharFormat,)


class TextImageFormat(QtGui.QTextImageFormat):
    def __bool__(self):
        return self.isValid()


if __name__ == "__main__":
    fmt = TextImageFormat()
    print(bool(fmt))
